#ifndef MECIFLIB
#define MECIFLIB
/********************************************************HP*03-5310*********
*
* $Author: prakt_fw $
* $Date: 92/12/03 10:38:11 $
* $Revision: 1.10 $
*
****************************************************************************
*
* CONTENTS : meciflib.h - header file for MECIF library functions
*
***************************************************************************


		(C) Copyright Hewlett-Packard GmbH 1992

			  All rights reserved


     Hewlett-Packard makes no warranty of any kind with regard to this 
     software, including, but not limited to, the implied warranties of 
     merchantability and fitness for a particular purpose.
     Hewlett-Packard shall not be liable for errors contained herein or
     direct, indirect, special, incidental or consequential damages in 
     connection with the furnishing, performance or use of this material.

*/



/*-SYSTEM INCLUDES--------------------------------------------------------*/


/*-MODULE INCLUDES--------------------------------------------------------*/

#include <pc_mach.h>
#include <pc_spi.h>
#include <mecif.h>

/*-EXTERNAL FUNCTIONS-----------------------------------------------------*/


/*-EXTERNAL VARIABLES-----------------------------------------------------*/


/*-DEFINES----------------------------------------------------------------*/

#define MAX_LINES       400		/* defines for the TW stuff */
#define NU_LINES        15

#define MAX_SAMPLES	64	/* max # of samples in CookedWave type */

#define MAX_HW_PORTS	4 		/* max # of hardware ports */
#define MAX_SERVERS	20		/* max # of MECIF servers  */
#define MAX_SESSIONS 	6		/* max # of sessions */
#define MAX_PAR_IDS 	128      	/* max # of MESSAGE IDs, contained *
					 * in a PAR_LIST_RSP command,      *
					 * storeable in a session          */
#define MAX_MISCL_IDS 	8      		/* max # of MESSAGE IDs, contained *
					 * in a MISC_LIST_RSP command,     *
					 * storeable in a session          */
#define MAX_QUERYL_IDS 	8      		/* max # of MESSAGE IDs, contained *
					 * in a QUERY_LIST_RSP command,    *
					 * storeable in a session          */

#define MAX_TUNE_IDS	10 		/* max # of tune ids per session */

			/* max # of source/tune IDs */
#define MAX_IDS		((MAX_TUNE_IDS+1) * MAX_SESSIONS)

#define SINGLE 		1
#define CONTINUOUS 	2

#define IDLE            0
#define NONE	       	-1

#define ENABLE		1
#define DISABLE		0


#define BIOS_TICKS_PER_DAY    1573040L


/* definition for the window size check */
#define FRAMES_PER_S		31

/* definition of the maximum response time for the DIRECTORY_REQ, *
 * MIRROR_REQ and the RAW_ENABLE_REQ commands which is 2 seconds  */
#define MAX_RSP_TIME		2

/* definition of error codes */
#define WINDOW_SIZE_OVERFLOW     1
#define TX_BUFFER_OVERFLOW       2
#define HW_HANDSHAKE_MALFUNCTION 3
#define MPB_FIFO_OVERFLOW        4
#define PARITY_ERROR             5
#define NOTIFIC_ABORTED		 6
#define LIFETICK_TIMEOUT 	 7
#define LIFETICK_FAILED  	 8
#define	NO_RSP_MIR_REQ		 9
#define	NO_RSP_DIR_REQ           10
#define	NO_RSP_RAWEN_REQ         11
#define	NO_RSP_CON_REQ		 12
#define	NO_RSP_DISCON_REQ	 13
#define MESSAGE_LOST             14
#define INVALID_COMMAND          15
#define RS232_RESTART_OV         20
#define RS232_RESTART_OTHER      21

/* definition of return codes */
#define FAIL			0
#define SUCCESS			1
#define OUT_OF_WINDOW_SIZE	-1
#define OUT_OF_TXBUFFER		-2
#define NOT_ENOUGH_MEMORY	-3
#define INCOMPATIBLE_SERVER	-4
#define BAUDRATE_WRONG          -2
#define PORT_WRONG              -1
/* definiton of states for the lifetick feature */
#define LIFETICK_SENT		3
#define LIFETICK_REC		4
#define LIFETICK_SENT_FAIL	5
#define MIRROR			6
#define DIRECTORY		9
#define RAW_ENABLE		10
#define CON_REQ_SENT		11
#define DISCON_REQ_SENT		12

/* definiton of states for the tune service */
#define TUNE_ATTEMPT		1
#define TUNED			2
#define SINGLE_TUNED    	3
#define LOW 			1
#define HIGH 			2


#ifdef _MSC_VER
#define setvect  _dos_setvect
#define getvect  _dos_getvect
#define enable   _enable
#define disable  _disable
#define outportb outp
#define inportb  (u_8)inp
#endif

/*-TYPEDEFS---------------------------------------------------------------*/

/* definition of a structure used to hold the *
 * session's lifetick information             */
typedef struct {
u_16            life_status;
u_16		status;
i_32		lib_period;
i_32		time;
u_16		info[MIRROR_NUM];
} Mirror;

/* structure to hold the session's tune ids */
typedef struct {
u_16		num_of_ids;
u_16		tune_id[MAX_TUNE_IDS];
} TuneId;

/* stucture to hold the session's window control infos */
typedef struct {
u_16		check;
u_16		size;
u_16		count;
} Win;


/* structure to store a message id and its associated ASCII id */
typedef struct {
c_8		ascii_id[12];
MsgIdTyp	msg_id;
} IdentTyp;


/* list to store the MESSAGE IDs received *
 * in response to a PAR_LIST_REQ command  */
typedef struct {
u_16		num_of_par;
u_8		actual;
u_8		total;
IdentTyp	ident[MAX_PAR_IDS];
} ParList;

/* definition of a table containing further information on a parameter */
typedef struct {
u_16		mpb_hd;
u_8		level;
u_8		status;
} ParInfo;

/* definition of a structure to hold all information on a parameter */
typedef struct {
ParList		list;
ParInfo		info[MAX_PAR_IDS];
} ParTyp;




/* definition of a list to store the MESSAGE IDs   *
 * received in response to a MISC_LIST_REQ command */
typedef struct {
u_16		num_of_par;
u_8		actual;
u_8		total;
IdentTyp	ident[MAX_MISCL_IDS];
} MiscList;

/* definition of a structure to hold information on a misc message */
typedef struct {
MiscList	list;
ParInfo		info[MAX_MISCL_IDS];
} MiscTyp;



/* list to store the MESSAGE IDs received *
 * in response to a QUERY_LIST_REQ command  */
typedef struct {
u_16		num_of_par;
u_8		actual;
u_8		total;
u_16		query_id[MAX_QUERYL_IDS];
} QueryList;




/* definition of the session table */
typedef struct {
i_16		con_id;
u_16		port_id;
u_16		dest_id;
TuneId		tune_id;
Win		window;
u_16		tick_info[TICK_NUM];
i_32		window_time;
Mirror		mirror;
ParTyp		par;
MiscTyp		misc;
QueryList	query;
} Session;


/* definition of one server table */
typedef struct {
u_16		serv_id;
u_16		num_connections;
u_8		compat_low;
u_8		compat_high;
c_8		serv_descr[DESCR_CHAR];
} ServerTbl;

/* definition of a table of servers */
typedef struct {
u_16		num_of_serv;
u_8		actual;
u_8		total;
u_16		language;
ServerTbl       server_tbl[MAX_SERVERS];
} TblofServ;

/* definition of the port table */
typedef struct {
u_16		valid;
u_16		port_num;
u_16		port_id;
TblofServ	tos;
} PortTbl;

/* definition of the LibTuneList type */
typedef struct {
u_16		num_tune;
MsgIdTyp	*msg_id_ptr;
} LibTuneList;

/* definition of the Tune Info type */
typedef struct {
MsgIdTyp	msg_id;
u_8		rec_id;
u_8		app_rec_len;
u_16		*word_ptr;
} SpiTuneInfo;

/* definition of the Tune Type Info type */
typedef struct {
u_16		msg_type;
u_8		rec_id;
u_8		app_rec_len;
u_16		*word_ptr;
} SpiTuneTypeInfo;

/* definition of the SpiTuneList type */
typedef struct {
u_16		num_tune;
SpiTuneInfo	*tune_info_ptr;
} SpiTuneList;

/* definition of the Detune Request List type */
typedef struct {
u_16		num_of_par;
MsgIdTyp	*msg_id_ptr;
} DetuneList;

/* definition of the Tune/Detune Response List type */
typedef struct {
MsgIdTyp	msg_id;
u_16		rsp;
} RspInfo;

typedef struct {
u_16		num_of_par;
RspInfo		*info_ptr;
} RspList;

/* some typedefinitions for use with low level parsers */
typedef struct {
i_16		con_id;
MsgIdTyp	msg_id;
u_16		num_of_samples;
u_16		samples[MAX_SAMPLES];
} CookedWave;

typedef struct {
i_16		con_id;
MsgIdTyp	msg_id;
u_16		status;
u_16		label_code;
u_16		num_of_samples;
c_8		label[SPI_LABEL_LENGTH];
} WaveSupp;

typedef struct {
float		value;
u_16		label_code;
} NumValue;

typedef struct {
i_16		con_id;
MsgIdTyp	msg_id;
u_16		label_code;
c_8		label[SPI_LABEL_LENGTH];
u_16		status;
u_16		num_of_values;
NumValue	numeric[SPI_MAX_NUMERICS];
} Numeric;

typedef struct {
i_16		con_id;
u_16		source_id;
u_16		source_no;
u_16		label_code;
u_16		status;
c_8		unsptext[SPI_ALERT_LENGTH];
} Alert;

typedef struct {
i_16		con_id;
u_8		al_severity;
u_8		inop_severity;
u_8		mon_status;
u_8		al_suspended;
} AlStat;

typedef struct {
i_16		con_id;
MsgIdTyp	msg_id;
u_16		label_code;
c_8		label[SPI_LABEL_LENGTH];
u_16		enum_code;
c_8		enum_string[SPI_ENUM_LENGTH];
} EnumData;

typedef struct {
i_16		con_id;
MsgIdTyp	msg_id;
c_8		dev_name[CDD_DEV_NAME_LENGTH];
} DevName;

typedef struct {
i_16		con_id;
MsgIdTyp	msg_id;
u_16		num_of_words;
u_16		data_word[255];
} TransparentData;

/* definition of the return value type for rx_mecif() */
typedef struct {
i_16		con_id;
u_16		cmd;
union {
i_16	iret;
void	*pret;
}		ret;
} LibRet;


/*-GLOBAL FUNCTIONS-------------------------------------------------------*/

/* in module MECIFREQ */
i_16 		mirror_req(i_16 con_id,u_16 port_id,u_8 num_of_info,
							u_16 *info_ptr);
i_16		directory_req(u_16 port_id);
i_16 		raw_enable_req(u_16 port_id,u_16 password1,u_16 password2);
i_16 		connect_req(u_16 port_id,u_16 server_id,u_16 lib_tick_period,
			    u_16 cms_tick_period,u_16 win_control);
i_16   		tick_req(i_16 con_id,u_8 tick_info_num,u_16 *tick_info_ptr);
i_16		par_list_req(i_16 con_id);
i_16		misc_list_req(i_16 con_id);
i_16		query_list_req(i_16 con_id);
i_16		tune_lib_req(i_16 con_id,LibTuneList *data_ptr,i_16 mode);
i_16		tune_spi_req(i_16 con_id,SpiTuneList *data_ptr,i_16 mode);
i_16		tune_lib_type_req(i_16 con_id,u_16 msg_type,i_16 mode);
i_16		tune_spi_type_req
			(i_16 con_id,SpiTuneTypeInfo *data_ptr,i_16 mode);
i_16 		detune_req(i_16 con_id,DetuneList *data_ptr);
i_16		detune_all_req(i_16 con_id,u_16 msg_type);
i_16		disconnect_req(i_16 con_id);
i_16 		query_req(i_16 con_id,i_16 tune_id,u_16 *query_id,
			  u_16 num_of_words,u_16 *query_cmd_ptr,u_8 rsp);

/* in module MECIFRSP */
i_16 		mirror_rsp(MirrorRsp *rmptr,Session **session_ptr_p);
TblofServ	*directory_rsp(DirRsp *rmptr,Session **session_ptr_p);
i_16 		raw_enable_rsp(RawEnRsp *rmptr,Session **session_ptr_p);
i_16 		connect_rsp(ConRsp *rmptr,Session **session_ptr_p);
i_16 		tick_rsp(TickRsp *rmptr,Session **session_ptr_p);
ParList		*par_list_rsp(ListRsp *rmptr,Session **session_ptr_p);
MiscList	*misc_list_rsp(MListRsp *rmptr,Session **session_ptr_p);
QueryList	*query_list_rsp(QListRsp *rmptr,Session **session_ptr_p);
RspList		*tune_rsp(TuneRsp *rmptr,Session **session_ptr_p);
RspList 	*tune_type_rsp(TuneTypeRsp *rmptr,Session **session_ptr_p);
RspList		*detune_rsp(DetuneRsp *rmptr,Session **session_ptr_p);
i_16 		detune_all_rsp(DetAllRsp *rmptr,Session **session_ptr_p);
i_16 		disconnect_rsp(DisconRsp *rmptr,Session **session_ptr_p);
i_16 		cms_mirror_rsp(MirrorReq *rmptr,Session **session_ptr_p);
i_16 		cms_tick_rsp(TickReq *rmptr,Session **session_ptr_p);
void 		rx_tune_data(u_16 *rmptr,Session **session_ptr_p);


/* in module MECIFOP */
u_16		init_rs232port(u_16 port,u_16 baudrate);
i_16		close_port(u_16 port_id);
Session 	**get_session(void);
Session 	**find_session(i_16 con_id);
void 		free_session(Session **poi);
PortTbl 	*get_port_tbl(u_16 port);
PortTbl 	*find_port_tbl(u_16 port_id);
void 		free_port_tbl(PortTbl *port_ptr);
i_16 		set_tune_id(i_16 con_id);
i_16 		search_info(Session **session_ptr_p,MsgIdTyp *in_id_ptr,
						ParInfo **out_info_ptr);
void		lifetick(void);
LibRet		*rx_mecif(void);
char     *get_lib_revision(void);


/* in module RS232 */
i_16  		mecif_init(u_16 port,u_16 baud);
i_16  		write_msg(u_16 port,u_16 *mptr);
u_16 		*read_msg(u_16 port);
void 		mecif_end(u_16 port);

/* in module C16TOC8 */
void c16_to_c8(c_16 *src_ptr,c_8 *dest_ptr,u_16 num_of_char);


/* to be defined by user (dummies in module DUMMY) */
void rx_spi_cw(SpiCwFixed64 *rmptr,Session **session_ptr_p,MsgIdTyp *id_ptr);
void rx_spi_ws(SpiWsFixed *rmptr,Session **session_ptr_p,MsgIdTyp *id_ptr);
void rx_spi_nu(SpiNuFixed *rmptr,Session **session_ptr_p,MsgIdTyp *id_ptr);
void rx_spi_unspec(u_16 *rmptr,Session **session_ptr_p,MsgIdTyp *id_ptr);
void rx_error(u_16 error_code,i_16 con_id);
void rx_lib_cw(CookedWave *cw_ptr);
void rx_lib_ws(WaveSupp *ws_ptr);
void rx_lib_nu(Numeric *nu_ptr);
void rx_lib_al(Alert *unsp_ptr);
void rx_lib_inop(Alert *unsp_ptr);
void rx_lib_alstat(AlStat *alstat_ptr);
void rx_lib_enum(EnumData *enum_ptr);
void rx_lib_devname(DevName *name_ptr);
void rx_lib_transp(TransparentData *data_ptr);
void rx_query_req(QueryReq *rmptr,Session **session_ptr_p);
void rx_query_rsp(QueryRsp *rmptr,Session **session_ptr_p);
void rx_unspec(u_16 *rmptr,Session **session_ptr_p);


#endif


