/********************************************************HP*03-5310*********
*
* $Author: prakt_fw $
* $Date: 92/12/03 10:37:33 $
* $Revision: 1.10 $
*
****************************************************************************
*
* CONTENTS : mecifop.c - mecif library operating functions
*
***************************************************************************


		(C) Copyright Hewlett-Packard GmbH 1992

			  All rights reserved


     Hewlett-Packard makes no warranty of any kind with regard to this 
     software, including, but not limited to, the implied warranties of
     merchantability and fitness for a particular purpose.
     Hewlett-Packard shall not be liable for errors contained herein or
     direct, indirect, special, incidental or consequential damages in 
     connection with the furnishing, performance or use of this material.

*/



/*-SYSTEM INCLUDES--------------------------------------------------------*/

#include <stdlib.h>
#include <dos.h>

/*-MODULE INCLUDES--------------------------------------------------------*/

#include <pc_mach.h>
#include <mecif.h>
#include <meciflib.h>

/*-EXTERNAL FUNCTIONS-----------------------------------------------------*/

/*-EXTERNAL VARIABLES-----------------------------------------------------*/

/* ch: 12.5.92 (BS) */
extern i_16 tune_type_busy_flag;
extern u_8  tune_type_flag;
extern RspList tune_type_rsp_list;
extern u_8  tune_type_actual;



/*-DEFINES----------------------------------------------------------------*/

#define ID_OFFSET		100
#define MAX_MSG_TYPES		4

#define CHECK_PORT(port) if ((port >= MAX_HW_PORTS))\
			 return(PORT_WRONG)

/*-TYPEDEFS---------------------------------------------------------------*/

/*-GLOBAL FUNCTIONS-------------------------------------------------------*/

/*-GLOBAL VARIABLES-------------------------------------------------------*/

PortTbl		confport_tbl[MAX_HW_PORTS];
Session		*session_ptr[MAX_SESSIONS];


/* ch: 12.5.92 (BS) */
/* adress of the bios tick counter , changed clocktick to *bios_clocktick
   allow read access only to the BIOS clocktick variable!!!               */
const i_32 far *bios_clocktick= (long far *) 0x0040006C;



/*-STATIC FUNCTIONS-------------------------------------------------------*/

static i_16 		get_id(void);
static void 		free_id(i_16 id);


/*-STATIC VARIABLES-------------------------------------------------------*/

static PortTbl		*rx_port_ptr = confport_tbl;
static LibRet near	resp;
static u_16 near  	id_tbl[MAX_IDS];
static u_16             *id_tbl_ptr = id_tbl;


/* actual revision of the library:                                        */
static char     *lib_revision="MECIF-LIB 1.10";




/***************************************************************************
*
* ROUTINE:init_rs232port
*
* WRITTEN BY: Juergen Barthel                         ON:25.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine calls the RS232 initialisation routine
*
****************************************************************************
* RETURNED VALUE:on succees it returns a pointer to a port table, else NULL
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
u_16
init_rs232port(u_16 port,u_16 baudrate)
{
PortTbl	*port_ptr;
i_16	ret;

/* check if port can be configured */
CHECK_PORT(port);

/* check if port can be configured */
if ((port_ptr = get_port_tbl(port)) != NULL)
    {
    /* configure the port */
    if ((ret = mecif_init(port,baudrate)) == SUCCESS)
	{
	port_ptr->port_id = port;
	return(port_ptr->port_id);
	}
    else
	{
	/* configuration failed, free the port table */
	free_port_tbl(port_ptr);
	return(ret);
	}
    }
else
    return(PORT_WRONG);
}

/***************************************************************************
*
* ROUTINE:close_port
*
* WRITTEN BY: Juergen Barthel                         ON:01.05.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine resets all configured ports to their previous
*		configuration
*
****************************************************************************
* RETURNED VALUE:none
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
close_port(u_16 port_id)
{
PortTbl *port_ptr;

/* if port is configured call mecif_end for this port to reset  *
 * the RS232 to its old status and clear all port table entries */
if ((port_ptr = find_port_tbl(port_id)) != NULL)
    {
    mecif_end(port_ptr->port_num);
    free_port_tbl(port_ptr);
    return(SUCCESS);
    }
else
    {
    return(FAIL);
    }
}


/***************************************************************************
*
* ROUTINE:get_session
*
* WRITTEN BY: Juergen Barthel                         ON:09.07.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:  this routine allocates a session and itializes all session
*		table entries
*
****************************************************************************
* RETURNED VALUE:on success it returns a pointer to the desired
*		 sessionpointer else NULL
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
* session_ptr      r   vector of pointers to sessions
****************************************************************************/
Session **get_session(void)
{
Session		**session_ptr_p;
u_8		found = FALSE;
u_8		i;

/* search for a free session tble */
session_ptr_p = session_ptr;
for (i = 0;i < MAX_SESSIONS;i++)
    {
    if (*session_ptr_p == NULL)
	{
	found = TRUE;
        break;
	}
    session_ptr_p++;
    }

/* if no free session is available return NULL */
if (found == FALSE)
    return(NULL);

*session_ptr_p = (Session *)malloc(sizeof(Session));
if (*session_ptr_p == NULL)
    return(NULL);

/* clear all session table entrys ------------*/

(*session_ptr_p)->port_id = NONE;
(*session_ptr_p)->dest_id = 0;

for (i = 0;i < MAX_TUNE_IDS;i++)
    (*session_ptr_p)->tune_id.tune_id[i] = 0;

(*session_ptr_p)->window.check  = DISABLE;
(*session_ptr_p)->window.size   = 0;
(*session_ptr_p)->window.count  = 0;
(*session_ptr_p)->mirror.status = DISABLE;
(*session_ptr_p)->mirror.life_status = DISABLE;

(*session_ptr_p)->mirror.time   = *bios_clocktick;
(*session_ptr_p)->window_time   = *bios_clocktick;

for (i = 0;i < MAX_PAR_IDS;i++)
    {
    (*session_ptr_p)->par.info[i].mpb_hd = 0;
    (*session_ptr_p)->par.info[i].status = IDLE;
    }
for (i = 0;i < MAX_MISCL_IDS;i++)
    {
    (*session_ptr_p)->misc.info[i].mpb_hd = 0;
    (*session_ptr_p)->misc.info[i].status = IDLE;
    }
(*session_ptr_p)->par.list.num_of_par  = 0;
(*session_ptr_p)->par.list.actual      = 0;
(*session_ptr_p)->par.list.total       = 0;
(*session_ptr_p)->misc.list.num_of_par = 0;
(*session_ptr_p)->misc.list.actual     = 0;
(*session_ptr_p)->misc.list.total      = 0;
(*session_ptr_p)->query.num_of_par     = 0;
(*session_ptr_p)->query.actual         = 0;
(*session_ptr_p)->query.total 	       = 0;


/* get the necessary session ids and check if *
 * all session ids are valid else release the *
 * session and the ids again and return NULL  */
if (((*session_ptr_p)->con_id = get_id()) == FAIL)
    {
    free(*session_ptr_p);
    *session_ptr_p = NULL;
    return(NULL);
    }
if (((*session_ptr_p)->tune_id.tune_id[0] = get_id()) == FAIL)
    {
    free_id((*session_ptr_p)->con_id);
    free(*session_ptr_p);
    *session_ptr_p = NULL;
    return(NULL);
    }
(*session_ptr_p)->tune_id.num_of_ids = 1;

/* return pointer to the new session pointer */
return(session_ptr_p);
}

/***************************************************************************
*
* ROUTINE:find_session
*
* WRITTEN BY: Juergen Barthel                         ON:18.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine finds the coresponding session to a given
*		connection ID
*
****************************************************************************
* RETURNED VALUE:on success it returns a pointer to the desired
*		 sessionpointer else NULL
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
* session_ptr      r   vector of pointers to sessions
****************************************************************************/
Session **find_session(i_16 con_id)
{
Session		**session_ptr_p;
u_16		*tune_id_ptr;
u_8		i;
u_16		j;

session_ptr_p = &session_ptr[0];

for (i = 0;i < MAX_SESSIONS;i++)
    {
    if (*session_ptr_p != NULL)
	{
	if (con_id == (*session_ptr_p)->con_id)
	    {
	    return(session_ptr_p);
	    }
	tune_id_ptr = (*session_ptr_p)->tune_id.tune_id;
	for (j = 0;j < (*session_ptr_p)->tune_id.num_of_ids;j++)
	    {
	    if ( (u_16)con_id == *tune_id_ptr++)
		{
		return(session_ptr_p);
		}
	    }
	}
    session_ptr_p++;
    }
return(NULL);
}

/***************************************************************************
*
* ROUTINE:free_session
*
* WRITTEN BY: Juergen Barthel                         ON:04.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine frees a previously allocated session table
*
****************************************************************************
* RETURNED VALUE:none
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
void free_session(Session **poi)
{
u_16	i;

if (*poi == NULL)
    return;

/* ch: 12.5.92 (BS) */
if ((*poi)->con_id == tune_type_busy_flag)
    {
    tune_type_actual = 0;
    tune_type_busy_flag = FALSE;
    tune_type_flag = IDLE;
    tune_type_rsp_list.num_of_par = 0;
    }

free_id((*poi)->con_id);

for (i = 0;i < (*poi)->tune_id.num_of_ids;i++)
    {
    free_id((*poi)->tune_id.tune_id[i]);
    }

free(*poi);
*poi = NULL;
}

/***************************************************************************
*
* ROUTINE:get_port_tbl
*
* WRITTEN BY: Juergen Barthel                         ON:15.05.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine gets a free port table
*
****************************************************************************
* RETURNED VALUE:pointer to a free port table else NULL
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
PortTbl *get_port_tbl(u_16 port)
{
PortTbl	*port_ptr;
u_8	i;


/* check if port is already configured */
port_ptr = confport_tbl;

for (i = 0;i < MAX_HW_PORTS;i++)
    {
    if (port_ptr->valid == TRUE)
	{
	if (port_ptr->port_num == port)
	    {
	    return(NULL);
	    }
	}
    port_ptr++;
    }

/* find a free port table */
port_ptr = confport_tbl;

for (i = 0;i < MAX_HW_PORTS;i++)
    {
    if (port_ptr->valid == FALSE)
	{
	port_ptr->valid = TRUE;
	port_ptr->port_num = port;
	return(port_ptr);
	}
    port_ptr++;
    }
return(NULL);
}

/***************************************************************************
*
* ROUTINE:find_port_tbl
*
* WRITTEN BY: Juergen Barthel                         ON:15.05.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine finds the coresponding port table to a given
*	      port_id
*
****************************************************************************
* RETURNED VALUE:pointer to the desired port table else NULL
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
PortTbl *find_port_tbl(u_16 port_id)
{
PortTbl	*port_ptr;
u_8	i;


/* search and return the port table */
port_ptr = confport_tbl;

for (i = 0;i < MAX_HW_PORTS;i++)
    {
    if (port_ptr->valid == TRUE)
	{
	if (port_ptr->port_id == port_id)
	    {
	    return(port_ptr);
	    }
	}
    port_ptr++;
    }
return(NULL);
}

/***************************************************************************
*
* ROUTINE:free_port_tbl
*
* WRITTEN BY: Juergen Barthel                         ON:09.07.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine resets a port table to its default
*
****************************************************************************
* RETURNED VALUE:none
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
void free_port_tbl(PortTbl *port_ptr)
{
u_8 	i;

if (port_ptr == NULL)
    return;

port_ptr->valid 	  = FALSE;
port_ptr->tos.num_of_serv = 0;
port_ptr->tos.actual 	  = 0;
port_ptr->tos.total 	  = 0;
for (i = 0;i < MAX_SERVERS;i++)
    {
    port_ptr->tos.server_tbl[i].serv_id 	= 0;
    port_ptr->tos.server_tbl[i].num_connections = 0;
    }
}

/***************************************************************************
*
* ROUTINE:get_id
*
* WRITTEN BY: Juergen Barthel                         ON:02.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine returns a unique ID
*
****************************************************************************
* RETURNED VALUE:unique i_16 value if available, else FAIL
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
* id_tbl	   w   table of unique IDs
****************************************************************************/
static i_16 get_id(void)
{
i_16	i;
i_16	id;


/* get unique id */
for (i = 0;i < MAX_IDS;i++)
    {
    if (*id_tbl_ptr == 0)
	{
	*id_tbl_ptr = 1;
	id = (i_16)(id_tbl_ptr - (u_16 *)id_tbl);
	id_tbl_ptr++;
	if (id_tbl_ptr == &id_tbl[MAX_IDS])
	    id_tbl_ptr = id_tbl;

	return(id + ID_OFFSET);
	}
    else
	{
	id_tbl_ptr++;
	if (id_tbl_ptr == &id_tbl[MAX_IDS])
	    id_tbl_ptr = id_tbl;
	}
    }
return(FAIL);
}

/***************************************************************************
*
* ROUTINE:free_id
*
* WRITTEN BY: Juergen Barthel                         ON:02.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine releases an ID which is no longer used
*
****************************************************************************
* RETURNED VALUE:none
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
* id_tbl           w   table of unique IDs
****************************************************************************/
static void free_id(i_16 id)
{
id_tbl[id - ID_OFFSET] = 0;
}

/***************************************************************************
*
* ROUTINE:set_tune_id
*
* WRITTEN BY: Juergen Barthel                         ON:28.08.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine enters a unique tune ID in the session table and
*	      returns this unique tune ID
*
****************************************************************************
* RETURNED VALUE:unique i_16 value if available, else FAIL
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
****************************************************************************/
i_16 set_tune_id(i_16 con_id)
{
Session	**session_ptr_p;
u_16   	tune_id_index;

/* check if connection is available */
if ((session_ptr_p = find_session(con_id)) == NULL)
    return(FAIL);

/* check if another tune id is possible */
if((*session_ptr_p)->tune_id.num_of_ids >= MAX_TUNE_IDS)
    return(FAIL);

tune_id_index = (*session_ptr_p)->tune_id.num_of_ids;
/* get unique id */
(*session_ptr_p)->tune_id.tune_id[tune_id_index] = get_id();

/* check if id is valid */
if ((*session_ptr_p)->tune_id.tune_id[tune_id_index] == FAIL)
    return(FAIL);

(*session_ptr_p)->tune_id.num_of_ids++;
return((*session_ptr_p)->tune_id.tune_id[tune_id_index]);
}

/***************************************************************************
*
* ROUTINE:search_info
*
* WRITTEN BY: Jrgen Barthel                          ON:27.09.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine adjusts a given pointer to a MsgInfo table by
*	      comparing a given msg id with the ones in the different tables
*	      in the session table
*
****************************************************************************
* RETURNED VALUE:1 on success else 0
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16 search_info(Session **session_ptr_p,MsgIdTyp *in_id_ptr,
						ParInfo **out_info_ptr)
{
IdentTyp	*ident_ptr;
u_16		i;

/* find the corresponding message info table */
*out_info_ptr = &((*session_ptr_p)->par.info[0]);
ident_ptr     = &((*session_ptr_p)->par.list.ident[0]);
for (i = 0;i < (*session_ptr_p)->par.list.num_of_par;
					i++,ident_ptr++,(*out_info_ptr)++)
    {
    if ((ident_ptr->msg_id.SourceId  == in_id_ptr->SourceId)  &&
	(ident_ptr->msg_id.ChannelId == in_id_ptr->ChannelId) &&
	(ident_ptr->msg_id.MsgType   == in_id_ptr->MsgType)   &&
	(ident_ptr->msg_id.ChannelNo == in_id_ptr->ChannelNo) &&
	(ident_ptr->msg_id.SourceNo  == in_id_ptr->SourceNo)  &&
	(ident_ptr->msg_id.Layer     == in_id_ptr->Layer)     )
	{
	return(1);
	}
    }
/* if not found check element misc */
*out_info_ptr = &((*session_ptr_p)->misc.info[0]);
ident_ptr     = &((*session_ptr_p)->misc.list.ident[0]);
for (i = 0;i < (*session_ptr_p)->misc.list.num_of_par;
					i++,ident_ptr++,(*out_info_ptr)++)
    {
    if ((ident_ptr->msg_id.SourceId  == in_id_ptr->SourceId)  &&
	(ident_ptr->msg_id.ChannelId == in_id_ptr->ChannelId) &&
	(ident_ptr->msg_id.MsgType   == in_id_ptr->MsgType)   &&
	(ident_ptr->msg_id.ChannelNo == in_id_ptr->ChannelNo) &&
	(ident_ptr->msg_id.SourceNo  == in_id_ptr->SourceNo)  &&
	(ident_ptr->msg_id.Layer     == in_id_ptr->Layer)     )
	{
	return(1);
	}
    }
return(0);
}

/***************************************************************************
*
* ROUTINE:lifetick
*
* WRITTEN BY: Juergen Barthel                         ON:12.06.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:
*
****************************************************************************
* RETURNED VALUE:
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
void lifetick(void)
{
Session		   **session_ptr_p;
u_8		      i;

/* ch: 12.5.92 (BS) */
register       long t_diff;


session_ptr_p = &session_ptr[0];

for (i = 0;i < MAX_SESSIONS;i++,session_ptr_p++)
    {
    if ((*session_ptr_p) != NULL)
	{

	/* ch: 12.5.92 (BS) */
	if ((t_diff = *bios_clocktick - (*session_ptr_p)->window_time) < 0)
	    t_diff += BIOS_TICKS_PER_DAY; /* handle 24h wrap around     */

	if(t_diff > 19)   /* for every 1040 ms          */
	    {
	    (*session_ptr_p)->window_time = *bios_clocktick;

	    /* adjust window size: subtract length of CONNECT_REQ and
		MIRROR_REQ because they are not handled by library */
	    (*session_ptr_p)->window.count =
		(FRAMES_PER_S * (*session_ptr_p)->window.size) - 22;

	    if ((*session_ptr_p)->mirror.status != DISABLE)
		{
		/* check mirror */

		/* ch: 12.5.92 (BS) */
		if ((t_diff = *bios_clocktick -
			      (*session_ptr_p)->mirror.time) < 0)
		    t_diff += BIOS_TICKS_PER_DAY;/* handle 24h wrap around */

		if (t_diff >= (*session_ptr_p)->mirror.lib_period)
		    {
		    switch ((*session_ptr_p)->mirror.status)
			{
			case LIFETICK_REC:
			/* send new mirror */
			if (mirror_req((*session_ptr_p)->con_id,
			(*session_ptr_p)->port_id,0,NULL) ==
					(*session_ptr_p)->con_id)
			    (*session_ptr_p)->mirror.status = LIFETICK_SENT;
			else
			    (*session_ptr_p)->mirror.status =
							 LIFETICK_SENT_FAIL;

			(*session_ptr_p)->mirror.time = *bios_clocktick;
			break;

			case LIFETICK_SENT_FAIL:
			/* send new tick */
			if (mirror_req((*session_ptr_p)->con_id,
			(*session_ptr_p)->port_id,0,NULL) ==
					(*session_ptr_p)->con_id)
			    {
			    (*session_ptr_p)->mirror.status = LIFETICK_SENT;
			    (*session_ptr_p)->mirror.time = *bios_clocktick;
			    }
			else
			    {
			    /* call rx_error */
			    rx_error(LIFETICK_TIMEOUT,
						(*session_ptr_p)->con_id);
/* abort notification */    disconnect_req((*session_ptr_p)->con_id);
			    free_session(session_ptr_p);
			    }
			break;

			case LIFETICK_SENT:
			/* send new tick */
			if (mirror_req((*session_ptr_p)->con_id,
			(*session_ptr_p)->port_id,0,NULL) ==
					(*session_ptr_p)->con_id)
			    {
			    (*session_ptr_p)->mirror.status=LIFETICK_TIMEOUT;
			    (*session_ptr_p)->mirror.time = *bios_clocktick;
			    }
			else
			    {
			    /* call rx_error */
			    rx_error(LIFETICK_TIMEOUT,
						(*session_ptr_p)->con_id);
/* abort notification */    disconnect_req((*session_ptr_p)->con_id);
			    free_session(session_ptr_p);
			    }
			break;

			case LIFETICK_TIMEOUT:
			/* call rx_error */
			rx_error(LIFETICK_TIMEOUT,(*session_ptr_p)->con_id);
/* abort notification */disconnect_req((*session_ptr_p)->con_id);
			free_session(session_ptr_p);
			break;

			case MIRROR:
			/* call rx_error */
			rx_error(NO_RSP_MIR_REQ,(*session_ptr_p)->con_id);
			free_session(session_ptr_p);
			break;

			case DIRECTORY:
			/* call rx_error */
			rx_error(NO_RSP_DIR_REQ,(*session_ptr_p)->con_id);
			free_session(session_ptr_p);
			break;

			case RAW_ENABLE:
			/* call rx_error */
			rx_error(NO_RSP_RAWEN_REQ,(*session_ptr_p)->con_id);
			free_session(session_ptr_p);
			break;

			case CON_REQ_SENT:
			/* call rx_error */
			rx_error(NO_RSP_CON_REQ,(*session_ptr_p)->con_id);
/* abort notification */disconnect_req((*session_ptr_p)->con_id);
			free_session(session_ptr_p);
			break;

			case DISCON_REQ_SENT:
			/* call rx_error */
			rx_error(NO_RSP_DISCON_REQ,(*session_ptr_p)->con_id);
			free_session(session_ptr_p);
			break;
			}
		    }
		}
	    }
	}
    }
}

/***************************************************************************
*
* ROUTINE:rx_mecif
*
* WRITTEN BY: Juergen Barthel                         ON:20.03.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION: this routine handels receiving MECIF messages
*
****************************************************************************
*
* RETURNED VALUE: pointer to a stus structure
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
* session_ptr      r   vector of pointers to sessions
*
****************************************************************************/
LibRet *rx_mecif(void)
{
Session		**session_ptr_p;
u_16		*rmptr;
u_8		i;


resp.con_id   = 0;
resp.cmd      = 0;
resp.ret.pret = NULL;

/* lifetick check */
lifetick();

/* try to read messages */
for (i = 0;(rx_port_ptr->valid == FALSE) && (i < MAX_HW_PORTS);i++)
    {
    rx_port_ptr++;
    if (rx_port_ptr > &(confport_tbl[MAX_HW_PORTS - 1]))
	rx_port_ptr = confport_tbl;
    }

if (i == MAX_HW_PORTS)
    return(&resp);

rmptr = read_msg(rx_port_ptr->port_id);

/* if a message was read find corresponding connection number */
if (rmptr != NULL)
    {
    session_ptr_p = find_session(((TransHd *)rmptr)->dest_id);

    if (session_ptr_p != NULL)
	{
	resp.con_id = (*session_ptr_p)->con_id;
	/* check if the received message is a control or tune data *
	 * if tune data call corresponding subroutine if control   *
	 * find out the controltype and call the corresponding     *
	 * control subroutine				           */
	if (((TransHd *)rmptr)->dest_id ==
	      (*session_ptr_p)->tune_id.tune_id[0])
	    {
	    rx_tune_data(rmptr,session_ptr_p);
	    }
	else if (((TransHd *)rmptr)->dest_id == (u_16)(*session_ptr_p)->con_id)
	    {
	    resp.cmd = ((ConReq *)rmptr)->cmd;
	    switch(((ConReq *)rmptr)->cmd)
		{
		case CONNECT_RSP:
		    resp.ret.iret = connect_rsp((ConRsp *)rmptr,
						session_ptr_p);
		break;

		case TICK_REQ:
		    resp.ret.iret = cms_tick_rsp((TickReq *)rmptr,
						 session_ptr_p);
		break;

		case TICK_RSP:
		    resp.ret.iret = tick_rsp((TickRsp *)rmptr,session_ptr_p);
		break;

		case OVERFLOW_NOTIFICATION:
		    rx_error(((OvflowNot *)rmptr)->error_code,
			     (*session_ptr_p)->con_id);
		break;

		case ABORT_NOTIFICATION:
		    /* call rx_error and free the session */
		    rx_error(NOTIFIC_ABORTED,(*session_ptr_p)->con_id);
		    free_session(session_ptr_p);
		break;

		case DISCONNECT_RSP:
		    resp.ret.iret = disconnect_rsp((DisconRsp *)rmptr,
						   session_ptr_p);
		break;

		case DIRECTORY_RSP:
		    resp.ret.pret = directory_rsp((DirRsp *)rmptr,
						  session_ptr_p);
		break;

		case PAR_LIST_RSP:
		    resp.ret.pret = par_list_rsp((ListRsp *)rmptr,
						 session_ptr_p);
		break;

		case TUNE_RSP:
		    resp.ret.pret = tune_rsp((TuneRsp *)rmptr,session_ptr_p);
		break;

		case SINGLE_TUNE_RSP:
		    resp.ret.pret = tune_rsp((TuneRsp *)rmptr,session_ptr_p);
		break;

		case DETUNE_RSP:
		    resp.ret.pret = detune_rsp((DetuneRsp *)rmptr,
						session_ptr_p);
		break;

		case DETUNE_ALL_RSP:
		    resp.ret.iret = detune_all_rsp((DetAllRsp *)rmptr,
						   session_ptr_p);
		break;

		case QUERY_REQ:
		    rx_query_req((QueryReq *)rmptr,session_ptr_p);
		break;

		case QUERY_RSP:
		    rx_query_rsp((QueryRsp *)rmptr,session_ptr_p);
		break;

		case QUERY_LIST_RSP:
		    resp.ret.pret = query_list_rsp((QListRsp *)rmptr,
						session_ptr_p);
		break;

		case MIRROR_REQ:
		    resp.ret.iret = cms_mirror_rsp((MirrorReq *)rmptr,
						 session_ptr_p);
		break;

		case MIRROR_RSP:
		    resp.ret.iret = mirror_rsp((MirrorRsp *)rmptr,
						session_ptr_p);
		break;

		case MISC_LIST_RSP:
		    resp.ret.pret = misc_list_rsp((MListRsp *)rmptr,
						 session_ptr_p);
		break;

		case RAW_ENABLE_RSP:
		    resp.ret.iret = raw_enable_rsp((RawEnRsp *)rmptr,
						session_ptr_p);
		break;

		case TUNE_TYPE_RSP:
		    resp.ret.pret = tune_type_rsp((TuneTypeRsp *)rmptr,
						session_ptr_p);
		break;

		case SINGLE_TUNE_TYPE_RSP:
		    resp.ret.pret = tune_type_rsp((TuneTypeRsp *)rmptr,
						session_ptr_p);
		break;

		default :
		    rx_error(INVALID_COMMAND, (*session_ptr_p)->con_id);
		    resp.ret.pret = rmptr;
		break;
		}
	    }
	else
	    {
	    /* if the dest_id is an other tune_id call rx_unspec */
	    rx_unspec(rmptr,session_ptr_p);
	    }
	}
    }

rx_port_ptr++;
if (rx_port_ptr > &(confport_tbl[MAX_HW_PORTS]))
    rx_port_ptr = confport_tbl;

return(&resp);
}



/*
/----------------------------------------------------------------------------\
| Modul Name :	G E T _ R E V I S I O N
|
|    Edition : AUG/1992    BOS
|
| Description: returns a char ptr to the actual library revision
|
| Parameters :	
|	
|    Returns : char ptr to a string 
|
|~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
| External References:
|
| Identifier      r/w  Comment
|-----------------------------------------------------------------------------
| lib_revision    r	     
\---------------------------------------------------------------------------*/
char  *get_lib_revision (void)
{
   return (lib_revision);
}

