/********************************************************HP*03-5310*********
*
* $Author: prakt_fw $
* $Date: 92/12/03 10:37:38 $
* $Revision: 1.10 $
*
****************************************************************************
*
* CONTENTS : mecifreq.c - function library with request functions
*
***************************************************************************


		(C) Copyright Hewlett-Packard GmbH 1992

			  All rights reserved


     Hewlett-Packard makes no warranty of any kind with regard to this 
     software, including, but not limited to, the implied warranties of 
     merchantability and fitness for a particular purpose.
     Hewlett-Packard shall not be liable for errors contained herein or
     direct, indirect, special, incidental or consequential damages in 
     connection with the furnishing, performance or use of this material.

*/



/*-SYSTEM INCLUDES--------------------------------------------------------*/


/*-MODULE INCLUDES--------------------------------------------------------*/

#include <pc_mach.h>
#include <mecif.h>
#include <meciflib.h>
#include <database.h>


/*-EXTERNAL FUNCTIONS-----------------------------------------------------*/

/*-EXTERNAL VARIABLES-----------------------------------------------------*/

/* ch: 12.5.92 (BS) */
extern const long far *bios_clocktick;


/*-DEFINES----------------------------------------------------------------*/

/* changed: 18.2.92, because of different MICRO/TURBO definition */
#define	LIB_CLK_TCK		18.2


/*-TYPEDEFS---------------------------------------------------------------*/

/*-GLOBAL FUNCTIONS-------------------------------------------------------*/

/*-GLOBAL VARIABLES-------------------------------------------------------*/

u_8 			tune_type_flag = IDLE;

/* ch: 12.5.92 (BS) */
i_16 			tune_type_busy_flag = FALSE;

/*-STATIC FUNCTIONS-------------------------------------------------------*/

/*-STATIC VARIABLES-------------------------------------------------------*/

static DirReq near	dir_req;
static ConReq near	con_req;
static ListReq near	list_req;
static TuneReq near	tune_req;
static TuneTypeReq near	tune_type_req;
static DetuneReq near	det_req;
static DetAllReq near	det_all_req;
static TickReq near	tickreq;
static DisconReq near	discon_req;
static MirrorReq near	mir_req;
static MListReq near	mlist_req;
static QListReq near	qlist_req;
static RawEnReq near	raw_en_req;
static QueryReq	near	query;

static SpiTuneInfo near tune_info[MAX_TUNE_REQ_IDS];
static SpiTuneList near	tune_list;
static SpiTuneTypeInfo near tune_type_info;

/* set library tune flag to FALSE */
static u_8 near		lib_tune_flag = FALSE;



/**************************************************************************
*
* ROUTINE:mirror_req
*
* WRITTEN BY: Juergen Barthel                         ON:20.09.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine sends the MIRROR_REQ command to CMS
*
****************************************************************************
* RETURNED VALUE: a unique connection ID if the MIRROR_REQ was sent
*
*		  FAIL if the specified port is not configured or no session
*		       is available,
*
*                 else the return value of "write_msg()" which is:
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it
*
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
mirror_req(i_16 con_id,u_16 port_id,u_8 num_of_info,u_16 *info_ptr)
{
Session		**session_ptr_p;
i_16		ret;
u_8		i;

/* check if num_of_info is o.k. */
if (num_of_info > MIRROR_NUM)
    return(FAIL);

/* if con_id is 0 get a free session, build the MIRROR REQUEST *
 * command and send it to the DIRECTORY SERVER in the CMS      */
if (!con_id)
    {
    /* check if port has been configured yet */
    if (!find_port_tbl(port_id))
	return(FAIL);

    if ((session_ptr_p = get_session()) == NULL)
	return(FAIL);

    (*session_ptr_p)->port_id           = port_id;
    (*session_ptr_p)->dest_id           = DIR_SERVER;
    (*session_ptr_p)->mirror.status     = MIRROR;
    (*session_ptr_p)->mirror.lib_period = (i_32)(MAX_RSP_TIME * LIB_CLK_TCK);
    (*session_ptr_p)->mirror.time 	= *bios_clocktick;
    }

    /* check if connection is available */
else if ((session_ptr_p = find_session(con_id)) == NULL)
    {
    return(FAIL);
    }


/* set the MIRROR REQUEST command */
mir_req.trans_hd.trans_len = sizeof(MirrorReq)-(MIRROR_NUM*2)+(num_of_info*2);
mir_req.trans_hd.dest_id   = (*session_ptr_p)->dest_id;
mir_req.trans_hd.src_id    = (*session_ptr_p)->con_id;
mir_req.cmd 		   = MIRROR_REQ;

/* enter mirror information */
for (i = 0;i < num_of_info;i++)
    {
    mir_req.mirror_info[i] = (*session_ptr_p)->mirror.info[i] = *info_ptr++;
    }

if ((ret = write_msg((*session_ptr_p)->port_id,(u_16 *)&mir_req)) == SUCCESS)
    {
    return((*session_ptr_p)->con_id);
    }
else if ((*session_ptr_p)->mirror.status == MIRROR)
    {
    free_session(session_ptr_p);
    }

return(ret);
}

/**************************************************************************
*
* ROUTINE:directory_req
*
* WRITTEN BY: Juergen Barthel                         ON:02.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine sends the DIRECTORY_REQ command to CMS
*
****************************************************************************
* RETURNED VALUE: a unique connection ID if the DIRECTORY_REQ was sent
*
*		  FAIL if the specified port is not configured or no session
*		       is available,
*
*                 else the return value of "write_msg()" whitch is:
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it
*
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
directory_req(u_16 port_id)
{
PortTbl		*port_ptr;
Session		**session_ptr_p;
i_16		ret;

/* check if port has been configured yet */
if ((port_ptr = find_port_tbl(port_id)) == NULL)
    return(FAIL);

/* get a free session, build the DIRECTORY REQUEST command *
 * and send it to CMS                                      */
if ((session_ptr_p = get_session()) == NULL)
    return(FAIL);

/* set actual and total to 0 */
port_ptr->tos.actual = port_ptr->tos.total = 0;

/* set status of the new session */
(*session_ptr_p)->port_id 	    = port_id;
(*session_ptr_p)->mirror.status     = DIRECTORY;
(*session_ptr_p)->mirror.lib_period = (i_32)(MAX_RSP_TIME * LIB_CLK_TCK);
(*session_ptr_p)->mirror.time 	    = *bios_clocktick;

/* set the DIRECTORY REQUEST command */
dir_req.trans_hd.trans_len 	    = sizeof(DirReq);
dir_req.trans_hd.dest_id   	    = DIR_SERVER;
dir_req.trans_hd.src_id 	    = (*session_ptr_p)->con_id;
dir_req.cmd                	    = DIRECTORY_REQ;

if ((ret = write_msg(port_id,(u_16 *)&dir_req)) == SUCCESS)
    return((*session_ptr_p)->con_id);
else
    {
    free_session(session_ptr_p);
    return(ret);
    }
}

/***************************************************************************
*
* ROUTINE:raw_enable_req
*
* WRITTEN BY: Juergen Barthel                         ON:08.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine sends the RAW_ENABLE REQUEST command to CMS
*
****************************************************************************
* RETURNED VALUE: a unique connection ID if the RAW_ENABLE_REQ was sent
*
*		  FAIL if the specified port is not configured or no session
*		       is available,
*
*                 else the return value of "write_msg()" whitch is:
*
*		  SUCCESS if command will be transmitted,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it
*
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
raw_enable_req(u_16 port_id,u_16 password1,u_16 password2)
{
Session		**session_ptr_p;
i_16		ret;

/* check if port is configured */
if (!find_port_tbl(port_id))
    return(FAIL);

/* get a free session, build the RAW_ENABLE REQUEST command *
 * and send it to CMS                                    */
if ((session_ptr_p = get_session()) == NULL)
    return(FAIL);

/* set status of the new session */
(*session_ptr_p)->port_id 	    = port_id;
(*session_ptr_p)->mirror.status     = RAW_ENABLE;
(*session_ptr_p)->mirror.lib_period = (i_32)(MAX_RSP_TIME * LIB_CLK_TCK);
(*session_ptr_p)->mirror.time 	    = *bios_clocktick;

/* set the RAW_ENABLE REQUEST command */
raw_en_req.trans_hd.trans_len = sizeof(RawEnReq);
raw_en_req.trans_hd.dest_id   = (*session_ptr_p)->dest_id = DIR_SERVER;
raw_en_req.trans_hd.src_id    = (*session_ptr_p)->con_id;
raw_en_req.cmd                = RAW_ENABLE_REQ;
raw_en_req.password1 	      = password1;
raw_en_req.password2 	      = password2;

if ((ret = write_msg(port_id,(u_16 *)&raw_en_req)) == SUCCESS)
    return((*session_ptr_p)->con_id);
else
    {
    free_session(session_ptr_p);
    return(ret);
    }
}

/***************************************************************************
*
* ROUTINE:connect_req
*
* WRITTEN BY: Juergen Barthel                         ON:08.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine sends the CONNECT REQUEST command to CMS
*
****************************************************************************
* RETURNED VALUE: a unique connection ID if the CONNECT_REQ was sent
*
*		  FAIL if the specified port is not configured or no session
*		       is available,
*
*                 else the return value of "write_msg()" whitch is:
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it
*
*
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
connect_req(u_16 port_id,u_16 server_id,u_16 lib_tick_period,
	    u_16 cms_tick_period,u_16 win_control)
{
Session		**session_ptr_p;
i_16		ret;

/* check if port is configured */
if (!find_port_tbl(port_id))
    return(FAIL);

/* get a free session, build the CONNECT REQUEST command *
 * and send it to CMS                                    */
if ((session_ptr_p = get_session()) == NULL)
    return(FAIL);

(*session_ptr_p)->window.check = win_control;
(*session_ptr_p)->port_id      = port_id;

/* set the CONNECT REQUEST command */
con_req.trans_hd.trans_len = sizeof(ConReq);
con_req.trans_hd.dest_id   = (*session_ptr_p)->dest_id = server_id;
con_req.trans_hd.src_id    = (*session_ptr_p)->con_id;
con_req.cmd                = CONNECT_REQ;
con_req.tick_period        = cms_tick_period;

if (lib_tick_period != 0)
    {
    (*session_ptr_p)->mirror.lib_period =
         (i_32) (((i_32)lib_tick_period) * LIB_CLK_TCK);
    (*session_ptr_p)->mirror.life_status = ENABLE;
    }
else
    {
    (*session_ptr_p)->mirror.lib_period = (i_32)(MAX_RSP_TIME * LIB_CLK_TCK);
    (*session_ptr_p)->mirror.life_status = DISABLE;
    }

(*session_ptr_p)->mirror.status     = CON_REQ_SENT;
(*session_ptr_p)->mirror.time	    = *bios_clocktick;

if ((ret = write_msg(port_id,(u_16 *)&con_req)) == SUCCESS)
    return((*session_ptr_p)->con_id);
else
    {
    free_session(session_ptr_p);
    return(ret);
    }
}

/**************************************************************************
*
* ROUTINE:tick_req
*
* WRITTEN BY: Juergen Barthel                         ON:13.05.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine sends the TICK_REQ command to CMS
*
****************************************************************************
* RETURNED VALUE: FAIL if the specified connection does not exist or
*                      the amount of tick words is exceeded
*
*		  OUT_OF_WINDOW_SIZE if the TICK_REQ was not written to the
*				     outputbuffer due to exeeded win. size
*
*                 else the return value of "write_msg()" whitch is:
*
*		  SUCCESS if command will be transmitted,
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it
*
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
tick_req(i_16 con_id,u_8 tick_info_num,u_16 *tick_info_ptr)
{
Session		**session_ptr_p;
i_16		ret;
u_8		i;

/* check if num_of_info is o.k. */
if (tick_info_num > TICK_NUM)
    return(FAIL);

/* check if connection is available */
if ((session_ptr_p = find_session(con_id)) == NULL)
    return(FAIL);

/* set the TICK REQUEST command */
tickreq.trans_hd.trans_len = sizeof(TickReq)-(TICK_NUM*2)+(tick_info_num*2);
tickreq.trans_hd.src_id    = (*session_ptr_p)->con_id;
tickreq.trans_hd.dest_id   = (*session_ptr_p)->dest_id;
tickreq.cmd 		   = TICK_REQ;

/* check window size */
if ((*session_ptr_p)->window.check == ENABLE)
    {
    if ((*session_ptr_p)->window.count < tickreq.trans_hd.trans_len)
	return(OUT_OF_WINDOW_SIZE);
    }

/* enter tick information */
for (i = 0;i < tick_info_num;i++)
    {
    tickreq.tick_info[i] = (*session_ptr_p)->tick_info[i] = *tick_info_ptr++;
    }

ret = write_msg((*session_ptr_p)->port_id,(u_16 *)&tickreq);

if (ret == SUCCESS)
    {
    (*session_ptr_p)->window.count -= tickreq.trans_hd.trans_len;
    }

return(ret);
}

/***************************************************************************
*
* ROUTINE:par_list_req
*
* WRITTEN BY: Juergen Barthel                         ON:08.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine sends the PAR LIST REQUEST command to CMS
*
****************************************************************************
* RETURNED VALUE: FAIL if the specified connection does not exist
*
*		  OUT_OF_WINDOW_SIZE if the PAR_LIST_REQ was not written to
*				     the outputbuffer due to exceeded
*				     window size
*
*                 else the return value of "write_msg()" whitch is:
*
*		  SUCCESS if command will be transmitted,
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it
*
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
par_list_req(i_16 con_id)
{
Session **session_ptr_p;
i_16	ret;


/* check if connection is available */
if (((session_ptr_p = find_session(con_id)) == NULL) ||
    ((*session_ptr_p)->par.list.actual != (*session_ptr_p)->par.list.total))
    return(FAIL);


/* set the PAR_LIST REQUEST command and send it to CMS */
list_req.trans_hd.trans_len = sizeof(ListReq);
list_req.trans_hd.dest_id   = (*session_ptr_p)->dest_id;
list_req.trans_hd.src_id    = (*session_ptr_p)->con_id;
list_req.cmd 		    = PAR_LIST_REQ;

/* check window size */
if ((*session_ptr_p)->window.check == ENABLE)
    {
    if ((*session_ptr_p)->window.count < list_req.trans_hd.trans_len)
	return(OUT_OF_WINDOW_SIZE);

    }

/* set actual and total to 0 */
(*session_ptr_p)->par.list.actual = (*session_ptr_p)->par.list.total = 0;

ret = write_msg((*session_ptr_p)->port_id,(u_16 *)&list_req);

if (ret == SUCCESS)
    {
    (*session_ptr_p)->window.count -= list_req.trans_hd.trans_len;
    }

return(ret);

}

/***************************************************************************
*
* ROUTINE:misc_list_req
*
* WRITTEN BY: Juergen Barthel                         ON:08.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine sends the QUERY LIST REQUEST command to CMS
*
****************************************************************************
*
* RETURNED VALUE: FAIL if the specified connection does not exist
*
*		  OUT_OF_WINDOW_SIZE if the MISC_LIST_REQ was not written to
*				     the outputbuffer due to exeeded
*				     window size
*
*                 else the return value of "write_msg()" whitch is:
*
*		  SUCCESS if command will be transmitted,
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it
*
*
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
misc_list_req(i_16 con_id)
{
Session **session_ptr_p;
i_16	ret;


/* check if connection is available */
if (((session_ptr_p = find_session(con_id)) == NULL) ||
    ((*session_ptr_p)->misc.list.actual != (*session_ptr_p)->misc.list.total))
    return(FAIL);

/* set the MISC_LIST REQUEST command and send it to CMS    */
mlist_req.trans_hd.trans_len = sizeof(MListReq);
mlist_req.trans_hd.dest_id   = (*session_ptr_p)->dest_id;
mlist_req.trans_hd.src_id    = (*session_ptr_p)->con_id;
mlist_req.cmd 		     = MISC_LIST_REQ;

/* check window size */
if ((*session_ptr_p)->window.check == ENABLE)
    {
    if ((*session_ptr_p)->window.count < mlist_req.trans_hd.trans_len)
	return(OUT_OF_WINDOW_SIZE);

    }

/* set actual and total to 0 */
(*session_ptr_p)->misc.list.actual = (*session_ptr_p)->misc.list.total = 0;

ret = write_msg((*session_ptr_p)->port_id,(u_16 *)&mlist_req);

if (ret == SUCCESS)
    {
    (*session_ptr_p)->window.count -= mlist_req.trans_hd.trans_len;
    }

return(ret);

}

/***************************************************************************
*
* ROUTINE:query_list_req
*
* WRITTEN BY: Juergen Barthel                         ON:08.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine sends the QUERY LIST REQUEST command to CMS
*
****************************************************************************
*
* RETURNED VALUE: FAIL if the specified connection does not exist
*
*		  OUT_OF_WINDOW_SIZE if the QUERY_LIST_REQ was not written to
*				     the outputbuffer due to exeeded
*				     window size
*
*                 else the return value of "write_msg()" whitch is:
*
*		  SUCCESS if command will be transmitted,
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it
*
*
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
query_list_req(i_16 con_id)
{
Session **session_ptr_p;
i_16	ret;


/* check if connection is available */
if (((session_ptr_p = find_session(con_id)) == NULL) ||
    ((*session_ptr_p)->query.actual != (*session_ptr_p)->query.total))
    return(FAIL);

/* build the QUERY_LIST_REQ command and send it to CMS    */
qlist_req.trans_hd.trans_len = sizeof(QListReq);
qlist_req.trans_hd.dest_id   = (*session_ptr_p)->dest_id;
qlist_req.trans_hd.src_id    = (*session_ptr_p)->con_id;
qlist_req.cmd 		     = QUERY_LIST_REQ;

/* check window size */
if ((*session_ptr_p)->window.check == ENABLE)
    {
    if ((*session_ptr_p)->window.count < qlist_req.trans_hd.trans_len)
	return(OUT_OF_WINDOW_SIZE);

    }

/* set actual and total to 0 */
(*session_ptr_p)->query.actual = (*session_ptr_p)->query.total = 0;

ret = write_msg((*session_ptr_p)->port_id,(u_16 *)&qlist_req);

if (ret == SUCCESS)
    {
    (*session_ptr_p)->window.count -= qlist_req.trans_hd.trans_len;
    }

return(ret);

}

/***************************************************************************
*
* ROUTINE:tune_lib_req
*
* WRITTEN BY: Juergen Barthel                         ON:14.06.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this is the low level tune routine, it sends a
*		(SINGLE) TUNE REQUEST message to the CMS
*
****************************************************************************
*
* RETURNED VALUE: FAIL if a specified parameter is invalid
*
*		  OUT_OF_WINDOW_SIZE if the (SINGLE_)TUNE_REQ was not written
*				     to the outputbuffer due to exeeded
*				     window size
*
*                 else the return value of "write_msg()" whitch is:
*
*		  SUCCESS if command will be transmitted,
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it
*
*
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
tune_lib_req(i_16 con_id,LibTuneList *data_ptr,i_16 mode)
{
Session 	**session_ptr_p;
SpiTuneInfo	*tune_info_ptr;
MsgIdTyp	*from_id_ptr;
u_16		j;

/* check if connection is available and number of tune requests is valid */
session_ptr_p = find_session(con_id);
if ((session_ptr_p == NULL) || (data_ptr->num_tune > MAX_TUNE_REQ_IDS))
    return(FAIL);

tune_info_ptr = tune_list.tune_info_ptr = &tune_info[0];
from_id_ptr = data_ptr->msg_id_ptr;

for (j = 0;j < data_ptr->num_tune;j++,from_id_ptr++,tune_info_ptr++)
    {
    tune_info_ptr->msg_id      = *from_id_ptr;
    tune_info_ptr->app_rec_len = 0;

    switch(from_id_ptr->MsgType)
	{
	case TYP_SpiNU:
            /* transmit fixed & numeric record*/
	    tune_info_ptr->rec_id = SPI_NUMERIC;
	break;

	case TYP_SpiWS:
            /* transmit fixed part only */
	    tune_info_ptr->rec_id = 0xff;
	break;

	default:
            /* transmit complete message */
	    tune_info_ptr->rec_id = 0;
	break;
	}
    }
tune_list.num_tune = data_ptr->num_tune;

lib_tune_flag = TRUE;
return(tune_spi_req(con_id,&tune_list,mode));
}

/***************************************************************************
*
* ROUTINE:tune_spi_req
*
* WRITTEN BY: Juergen Barthel                         ON:09.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this is the high level tune routine, it sends a
*		(SINGLE) TUNE REQUEST for up to	MAX_TUNE_REQ message IDs
*		to the CMS
*
****************************************************************************
*
* RETURNED VALUE: FAIL if a specified parameter is invalid
*
*		  OUT_OF_WINDOW_SIZE if the (SINGLE_)TUNE_REQ was not written
*				     to the outputbuffer due to exeeded
*				     window size
*
*                 else the return value of "write_msg()" whitch is:
*
*		  SUCCESS if command will be transmitted,
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it
*
*
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
tune_spi_req(i_16 con_id,SpiTuneList *data_ptr,i_16 mode)
{
Session 	**session_ptr_p;
SpiTuneInfo	*tune_info_ptr;
ParInfo		*par_info_ptr;
u_16		*tune_req_info_ptr;
u_16		*word_ptr;
u_16		j;
u_8		i;
u_8		level;
i_16		ret;


if (lib_tune_flag == TRUE)
    {
    level = LOW;
    lib_tune_flag = FALSE;
    }
else
    level = HIGH;

session_ptr_p = find_session(con_id);
if ((session_ptr_p == NULL) || (data_ptr->num_tune > MAX_TUNE_REQ_IDS))
    return(FAIL);

switch(mode)
    {
    case SINGLE:
	tune_req.cmd = SINGLE_TUNE_REQ;
    break;

    case CONTINUOUS:
	tune_req.cmd = TUNE_REQ;
    break;

    default:
	return(FAIL);
    }
tune_req.trans_hd.trans_len = sizeof(TuneReq) - (MAX_TUNE_REQ * 2);
tune_req.trans_hd.dest_id   = (*session_ptr_p)->dest_id;
tune_req.trans_hd.src_id    = (*session_ptr_p)->con_id;
tune_req.tune_id 	    = (*session_ptr_p)->tune_id.tune_id[0];

tune_info_ptr     = data_ptr->tune_info_ptr;
tune_req_info_ptr = tune_req.tune_info;
for (j = 0;j < data_ptr->num_tune;j++,tune_info_ptr++)
    {
    /* find the corresponding message info table */
    /* if not in any list, if there is space in one of the lists, add *
     * the message Id there else return FAIL */
    if (search_info(session_ptr_p,&(tune_info_ptr->msg_id),&par_info_ptr)==0)
	{
	if ((*session_ptr_p)->par.list.num_of_par < MAX_PAR_IDS)
	    {
	    par_info_ptr = &((*session_ptr_p)->par.
				info[(*session_ptr_p)->par.list.num_of_par]);
	    (*session_ptr_p)->par.list.ident[(*session_ptr_p)->par.list.
	    num_of_par].msg_id = tune_info_ptr->msg_id;

	    ((*session_ptr_p)->par.list.num_of_par)++;
	    }
	else if ((*session_ptr_p)->misc.list.num_of_par < MAX_MISCL_IDS)
	    {
	    par_info_ptr = &((*session_ptr_p)->misc.info[
				    (*session_ptr_p)->misc.list.num_of_par]);
	    (*session_ptr_p)->misc.list.ident[(*session_ptr_p)->misc.list.
	    num_of_par].msg_id = tune_info_ptr->msg_id;

	    ((*session_ptr_p)->misc.list.num_of_par)++;
	    }

	else
	    return(FAIL);
	}

    par_info_ptr->level = level;
    par_info_ptr->status = TUNE_ATTEMPT;

    /* build the TUNE REQUEST command */
    /* enter tune info into tune request */
    tune_req.trans_hd.trans_len+=(sizeof(MsgIdTyp)+4);
    if (tune_req.trans_hd.trans_len <= sizeof(TuneReq))
	{
	/* enter Msg Id */
	*((MsgIdTyp *)tune_req_info_ptr) = tune_info_ptr->msg_id;
	tune_req_info_ptr+=(sizeof(MsgIdTyp)/2);

	/* enter Mpb Hd */
	*tune_req_info_ptr++ = 0;

	/* enter rec_id and app_rec_len */
	*((u_8 *)tune_req_info_ptr) = tune_info_ptr->rec_id;
	*(((u_8 *)tune_req_info_ptr)+1) = tune_info_ptr->app_rec_len;

	tune_req_info_ptr++;
	}
    else
	return(FAIL);

    tune_req.trans_hd.trans_len+=(tune_info_ptr->app_rec_len * 2);
    if (tune_req.trans_hd.trans_len <= sizeof(TuneReq))
	{
	/* enter words */
	word_ptr = tune_info_ptr->word_ptr;
	for (i = 0;i < tune_info_ptr->app_rec_len;i++)
	    *tune_req_info_ptr++ = *word_ptr++;
	}
    else
	return(FAIL);
    }

/* check window size */
if ((*session_ptr_p)->window.check == ENABLE)
    {
    if ((*session_ptr_p)->window.count < tune_req.trans_hd.trans_len)
	return(OUT_OF_WINDOW_SIZE);

    }

/* transfer complete tune_req message */
ret = write_msg((*session_ptr_p)->port_id,(u_16 *)&tune_req);

if (ret == SUCCESS)
    {
    (*session_ptr_p)->window.count -= tune_req.trans_hd.trans_len;
    }

return(ret);


}

/***************************************************************************
*
* ROUTINE:tune_lib_type_req
*
* WRITTEN BY: Juergen Barthel                         ON:10.02.92
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this is the low level tune routine, it sends a
*		(SINGLE) TUNE TYPE REQUEST message to the CMS
*
****************************************************************************
*
* RETURNED VALUE: FAIL if a specified parameter is invalid
*
*		  OUT_OF_WINDOW_SIZE if the (SINGLE_)TUNE_TYPE_REQ was not
*				     written to the outputbuffer due to
*				     exeeded window size
*
*                 else the return value of "write_msg()" whitch is:
*
*		  SUCCESS if command will be transmitted,
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it
*
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
tune_lib_type_req(i_16 con_id,u_16 msg_type,i_16 mode)
{
tune_type_info.msg_type      = msg_type;
tune_type_info.app_rec_len   = 0;
tune_type_info.word_ptr      = NULL;

switch(msg_type)
    {
    case TYP_SpiNU:
	/* transmit fixed & numeric record*/
	tune_type_info.rec_id = SPI_NUMERIC;
    break;

    case TYP_SpiWS:
	/* transmit fixed part only */
	tune_type_info.rec_id = 0xff;
    break;

    case TYP_SpiCW:
	/* transmit complete message */
	tune_type_info.rec_id = 0;
    break;

    case TYP_SpiMX:
	/* transmit complete message */
	tune_type_info.rec_id = 0;
    break;

    default:
	return(FAIL);
    break;
    }

tune_type_flag = LOW;
return(tune_spi_type_req(con_id,&tune_type_info,mode));
}

/***************************************************************************
*
* ROUTINE:tune_spi_type_req
*
* WRITTEN BY: Juergen Barthel                         ON:10.02.92
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this is the high level tune routine, it sends a
*		(SINGLE) TUNE TYPE REQUEST to the CMS
*
****************************************************************************
*
* RETURNED VALUE: FAIL if a specified parameter is invalid
*
*		  OUT_OF_WINDOW_SIZE if the (SINGLE_)TUNE_TYPE_REQ was not written
*				     to the outputbuffer due to exeeded
*				     window size
*
*                 else the return value of "write_msg()" whitch is:
*
*		  SUCCESS if command will be transmitted,
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it
*
*
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
tune_spi_type_req(i_16 con_id,SpiTuneTypeInfo *data_ptr,i_16 mode)
{
Session 	**session_ptr_p;
u_16		*tune_info_ptr;
u_16		*word_ptr;
i_16		ret;
u_16		j;


if (tune_type_busy_flag != FALSE)
    {
    return(FAIL);
    }

session_ptr_p = find_session(con_id);
if (session_ptr_p == NULL)
    return(FAIL);

switch(data_ptr->msg_type)
    {
    case TYP_SpiNU:
    case TYP_SpiWS:
    case TYP_SpiCW:
    case TYP_SpiMX:
	break;

    default:
	return(FAIL);
    }

switch(mode)
    {
    case SINGLE:
	tune_type_req.cmd = SINGLE_TUNE_TYPE_REQ;
    break;

    case CONTINUOUS:
	tune_type_req.cmd = TUNE_TYPE_REQ;
    break;

    default:
	return(FAIL);
    }
tune_type_req.trans_hd.trans_len=sizeof(TuneTypeReq)-(MAX_TUNE_TYPE_WORDS*2);
tune_type_req.trans_hd.dest_id  = (*session_ptr_p)->dest_id;
tune_type_req.trans_hd.src_id   = (*session_ptr_p)->con_id;
tune_type_req.tune_id 	        = (*session_ptr_p)->tune_id.tune_id[0];
tune_type_req.msg_type 	        = data_ptr->msg_type;
tune_type_req.rec_id 	        = data_ptr->rec_id;
tune_type_req.app_rec_len	= data_ptr->app_rec_len;


tune_info_ptr = tune_type_req.tune_info;
word_ptr      = data_ptr->word_ptr;

tune_type_req.trans_hd.trans_len+=(data_ptr->app_rec_len * 2);
if (tune_type_req.trans_hd.trans_len <= sizeof(TuneTypeReq))
    {
    for (j = 0;j < data_ptr->app_rec_len;j++)
	{
	*tune_info_ptr++ = *word_ptr++;
	}
    }
else
    {
    return(FAIL);
    }

/* check window size */
if ((*session_ptr_p)->window.check == ENABLE)
    {
    if ((*session_ptr_p)->window.count < tune_type_req.trans_hd.trans_len)
	return(OUT_OF_WINDOW_SIZE);
    }

/* transfer complete tune_type_req message */
ret = write_msg((*session_ptr_p)->port_id,(u_16 *)&tune_type_req);

if (ret == SUCCESS)
    {
    (*session_ptr_p)->window.count -= tune_type_req.trans_hd.trans_len;
   
   /* ch: 12.5.92 (BS) */
   tune_type_busy_flag = con_id;
    if (tune_type_flag == IDLE)
	{
	tune_type_flag = HIGH;
	}
    }

return(ret);
}

/***************************************************************************
*
* ROUTINE:detune_req
*
* WRITTEN BY: Juergen Barthel                         ON:09.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine sends the DETUNE REQUEST command for up to
*               MAX_DETUNE_REQ message IDs
*
****************************************************************************
*
* RETURNED VALUE: FAIL if a specified parameter is invalid
*
*		  OUT_OF_WINDOW_SIZE if the DETUNE_REQ was not written
*				     to the outputbuffer due to exeeded
*				     window size
*
*                 else the return value of "write_msg()" whitch is:
*
*		  SUCCESS if command will be transmitted,
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it
*
*
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16 detune_req(i_16 con_id,DetuneList *data_ptr)
{
Session 	**session_ptr_p;
ParInfo		*par_info_ptr;
MsgIdTyp	*detune_id_ptr;
DetuneInfo	*detune_info_ptr;
u_16		i;
i_16		ret;


/* get the corresponding session, build the DETUNE REQUEST *
 * command and send it to CMS                             */
session_ptr_p = find_session(con_id);

if ((!session_ptr_p) || (data_ptr->num_of_par > MAX_DETUNE_REQ))
    return(FAIL);

/* set the DETUNE REQUEST command */
det_req.trans_hd.trans_len = sizeof(DetuneReq)
			     - (MAX_DETUNE_REQ * sizeof(DetuneInfo))
			     + (data_ptr->num_of_par * sizeof(DetuneInfo));
det_req.trans_hd.dest_id   = (*session_ptr_p)->dest_id;
det_req.trans_hd.src_id    = (*session_ptr_p)->con_id;
det_req.cmd 		   = DETUNE_REQ;
det_req.tune_id 	   = (*session_ptr_p)->tune_id.tune_id[0];

/* enter all Msg Ids into the detune list */
detune_info_ptr = &(det_req.detune_info[0]);
detune_id_ptr   = data_ptr->msg_id_ptr;
for (i = 0;i < data_ptr->num_of_par;i++,detune_id_ptr++,detune_info_ptr++)
    {
    /* search for the coressponding Mpb Hd */
    if (search_info(session_ptr_p,detune_id_ptr,&par_info_ptr) == 1)
	{
	detune_info_ptr->mpb_hd = par_info_ptr->mpb_hd;
	detune_info_ptr->rsp    = 0;
	}
    else
	{
	/* don't send a DETUNE REQ command for a Msg Id that is not known */
	detune_info_ptr--;
	det_req.trans_hd.trans_len-=sizeof(DetuneInfo);
	}
    }

/* check window size */
if ((*session_ptr_p)->window.check == ENABLE)
    {
    if ((*session_ptr_p)->window.count < det_req.trans_hd.trans_len)
	return(OUT_OF_WINDOW_SIZE);

    }

ret = write_msg((*session_ptr_p)->port_id,(u_16 *)&det_req);

if (ret == SUCCESS)
    {
    (*session_ptr_p)->window.count -= det_req.trans_hd.trans_len;
    }

return(ret);

}

/***************************************************************************
*
* ROUTINE:detune_all_req
*
* WRITTEN BY: Juergen Barthel                         ON:09.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine sends the DETUNE ALL REQUEST command to the CMS
*
****************************************************************************
*
* RETURNED VALUE: FAIL if the specified connection ID is invalid
*
*		  OUT_OF_WINDOW_SIZE if the DETUNE_ALL_REQ was not written
*				     to the outputbuffer due to exeeded
*				     window size
*
*                 else the return value of "write_msg()" whitch is:
*
*		  SUCCESS if command will be transmitted,
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it
*
*
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16 detune_all_req(i_16 con_id,u_16 msg_type)
{
Session		**session_ptr_p;
i_16		ret;


/* get the corresponding session, build the DETUNE REQUEST *
 * command and send it to CMS                             */
if ((session_ptr_p = find_session(con_id)) == NULL)
    return(FAIL);

/* set the DETUNE ALL REQUEST command */
det_all_req.trans_hd.trans_len = sizeof(DetAllReq);
det_all_req.trans_hd.dest_id   = (*session_ptr_p)->dest_id;
det_all_req.trans_hd.src_id    = (*session_ptr_p)->con_id;
det_all_req.cmd 	       = DETUNE_ALL_REQ;
det_all_req.tune_id 	       = (*session_ptr_p)->tune_id.tune_id[0];
det_all_req.msg_type 	       = msg_type;

/* check window size */
if ((*session_ptr_p)->window.check == ENABLE)
    {
    if ((*session_ptr_p)->window.count < det_all_req.trans_hd.trans_len)
	return(OUT_OF_WINDOW_SIZE);

    }

ret = write_msg((*session_ptr_p)->port_id,(u_16 *)&det_all_req);

if (ret == SUCCESS)
    {
    (*session_ptr_p)->window.count -= det_all_req.trans_hd.trans_len;
    }

return(ret);

}

/***************************************************************************
*
* ROUTINE:disconnect_req
*
* WRITTEN BY: Juergen Barthel                         ON:09.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine sends the DISCONNECT REQUEST command to the CMS
*
****************************************************************************
*
* RETURNED VALUE: FAIL if the specified connection ID is invalid
*
*		  OUT_OF_WINDOW_SIZE if the DISCONNECT_REQ was not written
*				     to the outputbuffer due to exeeded
*				     window size
*
*                 else the return value of "write_msg()" whitch is:
*
*		  SUCCESS if command will be transmitted,
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it
*
*
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16 disconnect_req(i_16 con_id)
{
Session **session_ptr_p;
i_16	ret;


/* check if connection is available */
if ((session_ptr_p = find_session(con_id)) == NULL)
    return(FAIL);

/* build the DISCONNECT REQUEST command and send it to CMS    */
discon_req.trans_hd.trans_len = sizeof(DisconReq);
discon_req.trans_hd.dest_id   = (*session_ptr_p)->dest_id;
discon_req.trans_hd.src_id    = (*session_ptr_p)->con_id;
discon_req.cmd 		      = DISCONNECT_REQ;

/* check window size */
if ((*session_ptr_p)->window.check == ENABLE)
    {
    if ((*session_ptr_p)->window.count < discon_req.trans_hd.trans_len)
	return(OUT_OF_WINDOW_SIZE);

    }

if ((*session_ptr_p)->mirror.status == DISABLE)
    {
    (*session_ptr_p)->mirror.lib_period = (i_32)(MAX_RSP_TIME * LIB_CLK_TCK);
    }

(*session_ptr_p)->mirror.status     = DISCON_REQ_SENT;
(*session_ptr_p)->mirror.time	    = *bios_clocktick;

ret = write_msg((*session_ptr_p)->port_id,(u_16 *)&discon_req);

if (ret == SUCCESS)
    {
    (*session_ptr_p)->window.count -= discon_req.trans_hd.trans_len;
    }

return(ret);

}

/***************************************************************************
*
* ROUTINE:query_req
*
* WRITTEN BY: Juergen Barthel                         ON:28.08.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine sends the QUERY REQUEST command to CMS
*
****************************************************************************
*
* RETURNED VALUE: FAIL if a specified parameter is invalid
*
*		  OUT_OF_WINDOW_SIZE if the QUERY_REQ was not written
*				     to the outputbuffer due to exeeded
*				     window size
*
*                 else the return value of "write_msg()" whitch is:
*
*		  SUCCESS if command will be transmitted,
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it
*
*
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16 query_req(i_16 con_id,i_16 tune_id,u_16 *query_id,
	      u_16 num_of_words,u_16 *query_cmd_ptr,u_8 rsp)
{
Session **session_ptr_p;
u_16	*tune_id_ptr;
u_16	*cmd_ptr;
u_16	i;
i_16	ret;


/* check if connection is available */
if ((session_ptr_p = find_session(con_id)) == NULL)
    {
    return(FAIL);
    }

/* check if tune_id is in the session table */
tune_id_ptr = (*session_ptr_p)->tune_id.tune_id;
for (i = 0;i < (*session_ptr_p)->tune_id.num_of_ids;i++)
    {
    if (*tune_id_ptr++ == (u_16)tune_id)
	{
	break;
	}
    }
if (i == (*session_ptr_p)->tune_id.num_of_ids)
    {
    return(FAIL);
    }

/* build the QUERY_REQ command */
query.trans_hd.trans_len=sizeof(TransHd)+sizeof(u_16)+6+(2*num_of_words);
query.trans_hd.dest_id  = (*session_ptr_p)->dest_id;
query.trans_hd.src_id   = (*session_ptr_p)->con_id;
query.cmd		= QUERY_REQ;
query.query_id		= *query_id;
query.tune_id		= tune_id;
query.rsp		= rsp;

cmd_ptr = query.query_cmd;

while (num_of_words)
    {
    *cmd_ptr++ = *query_cmd_ptr++;
    num_of_words--;
    }

/* check window size */
if ((*session_ptr_p)->window.check == ENABLE)
    {
    if ((*session_ptr_p)->window.count < query.trans_hd.trans_len)
	return(OUT_OF_WINDOW_SIZE);

    }

/* send the QUERY REQUEST command to CMS */
ret = write_msg((*session_ptr_p)->port_id,(u_16 *)&query);

if (ret == SUCCESS)
    {
    (*session_ptr_p)->window.count -= query.trans_hd.trans_len;
    }

return(ret);

}
