/********************************************************HP*03-5310*********
*
* $Author: prakt_fw $
* $Date: 92/12/03 10:37:43 $
* $Revision: 1.10 $
*
****************************************************************************
*
* CONTENTS : mecifrsp.c - function library with response functions
*
***************************************************************************


		(C) Copyright Hewlett-Packard GmbH 1992

			  All rights reserved


     Hewlett-Packard makes no warranty of any kind with regard to this 
     software, including, but not limited to, the implied warranties of 
     merchantability and fitness for a particular purpose.
     Hewlett-Packard shall not be liable for errors contained herein or
     direct, indirect, special, incidental or consequential damages in 
     connection with the furnishing, performance or use of this material.

*/



/*-SYSTEM INCLUDES--------------------------------------------------------*/

#include <string.h>

/*-MODULE INCLUDES--------------------------------------------------------*/

#include <pc_mach.h>
#include <mecif.h>
#include <meciflib.h>
#include <database.h>

/*-EXTERNAL FUNCTIONS-----------------------------------------------------*/

extern void lparse_cw(SpiCwFixed64 *rmptr,Session **session_ptr_p,
		     MsgIdTyp *id_ptr);
extern void lparse_ws(SpiWsFixed *rmptr,Session **session_ptr_p,
		     MsgIdTyp *id_ptr);
extern void lparse_nu(SpiNuFixed *rmptr,Session **session_ptr_p,
		     MsgIdTyp *id_ptr);
extern void lparse_unspec(u_16 *rmptr,Session **session_ptr_p,
			 MsgIdTyp *id_ptr);
extern void lparse_alstat(u_16 *rmptr,Session **session_ptr_p);
extern void lparse_cddnu(SpiCddNuFixed *rmptr,Session **session_ptr_p,
			MsgIdTyp *id_ptr);
extern void lparse_cddset(SpiCddSetFixed *rmptr,Session **session_ptr_p,
			 MsgIdTyp *id_ptr);
extern void lparse_cdddev(SpiCddDevNameFixed *rmptr,Session **session_ptr_p,
			 MsgIdTyp *id_ptr);
extern void lparse_cddtran(SpiCddTransparent *rmptr,Session **session_ptr_p,
			  MsgIdTyp *id_ptr);

/*-EXTERNAL VARIABLES-----------------------------------------------------*/

/* changed by (BS) 12.5.92 */
extern const i_32 far *bios_clocktick;

extern u_8 		tune_type_flag;
extern i_16 		tune_type_busy_flag;

/*-DEFINES----------------------------------------------------------------*/

/*-TYPEDEFS---------------------------------------------------------------*/

/*-GLOBAL FUNCTIONS-------------------------------------------------------*/

/*-GLOBAL VARIABLES-------------------------------------------------------*/
RspList tune_type_rsp_list;
u_8	tune_type_actual = 0;

/*-STATIC FUNCTIONS-------------------------------------------------------*/

static void 		get_ascii_typ(u_16 msg_typ,c_8 *ptr);

/*-STATIC VARIABLES-------------------------------------------------------*/

static TickRsp near	cmstick_rsp;
static MirrorRsp near	mir_rsp;
static RspInfo near	rsp_info[MAX_TUNE_REQ_IDS];
static RspList near	rsp_list;
static RspInfo near     tune_type_rsp_info[MAX_TUNE_TYPE_WORDS];

/***************************************************************************
*
* ROUTINE: rx_tune_data
*
* WRITTEN BY: Juergen Barthel                         ON:21.03.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine checks the type of the received SPI message and
*	      calls the necessary subsequent routine
*
****************************************************************************
* RETURNED VALUE:none
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
void rx_tune_data(u_16 *rmptr,Session **session_ptr_p)
{
ParInfo		*par_info_ptr;
MsgIdTyp	*id_ptr = NULL;
u_16 		i;


/* find the corresponding parameter in the message info table */
par_info_ptr = &((*session_ptr_p)->par.info[0]);
for (i = 0;i < (*session_ptr_p)->par.list.num_of_par;i++,par_info_ptr++)
    {
    /* test auf tuned || single tuned sollte ueberfluessig sein */
    if((par_info_ptr->status==TUNED) || (par_info_ptr->status==SINGLE_TUNED))
	{
	if (par_info_ptr->mpb_hd == ((TuneData *)rmptr)->mpb_hd)
	    {
	    id_ptr = &((*session_ptr_p)->par.list.ident[i].msg_id);
	    break;
	    }
	}
    }
if (id_ptr == NULL)
    {
    par_info_ptr = &((*session_ptr_p)->misc.info[0]);
    for (i = 0;i < (*session_ptr_p)->misc.list.num_of_par;i++,par_info_ptr++)
	{
	if ((par_info_ptr->status == TUNED) ||
	    (par_info_ptr->status == SINGLE_TUNED))
	    {
	    if (par_info_ptr->mpb_hd == ((TuneData *)rmptr)->mpb_hd)
		{
		id_ptr = &((*session_ptr_p)->misc.list.ident[i].msg_id);
		break;
		}
	    }
	}
    }

if (id_ptr == NULL)
    return;

if (par_info_ptr->level == HIGH)
    {
    switch (id_ptr->MsgType)
	{
	case TYP_SpiCW:
	    rx_spi_cw((SpiCwFixed64 *)(rmptr + 3),session_ptr_p,id_ptr);
	break;

	case TYP_SpiWS:
	    rx_spi_ws((SpiWsFixed *)(rmptr + 3),session_ptr_p,id_ptr);
	break;

	case TYP_SpiNU:
	    rx_spi_nu((SpiNuFixed *)(rmptr + 3),session_ptr_p,id_ptr);
	break;

	default:
	    rx_spi_unspec(rmptr+3,session_ptr_p,id_ptr);
	break;
	}
    }
else
    {
    switch (id_ptr->MsgType)
	{
	case TYP_SpiCW:
	    lparse_cw((SpiCwFixed64 *)(rmptr+3),session_ptr_p,id_ptr);
	break;

	case TYP_SpiWS:
	    lparse_ws((SpiWsFixed *)(rmptr+3),session_ptr_p,id_ptr);
	break;

	case TYP_SpiNU:
	    lparse_nu((SpiNuFixed *)(rmptr+3),session_ptr_p,id_ptr);
	break;

	case TYP_SpiMX:
	    switch(((SpiCddNuFixed *)(rmptr + 3))->rec_type)
		{
		case SPI_CDD_NU_FIXED:
		    lparse_cddnu((SpiCddNuFixed *)(rmptr+3),
				session_ptr_p,id_ptr);
		break;

		case SPI_CDD_SET_FIXED:
		    lparse_cddset((SpiCddSetFixed *)(rmptr+3),
					 session_ptr_p,id_ptr);
		break;

		case SPI_CDD_DEV_NAME_FIXED:
		    lparse_cdddev((SpiCddDevNameFixed *)(rmptr+3),
				session_ptr_p,id_ptr);
		break;

		case SPI_CDD_TRANSPARENT_FIXED:
		    lparse_cddtran((SpiCddTransparent *)(rmptr+3),
				session_ptr_p,id_ptr);
		break;

		default:
		    rx_spi_unspec(rmptr + 3,session_ptr_p,id_ptr);
		break;
		}
	break;

	case TYP_Unspec:
	    switch(id_ptr->ChannelId)
		{
		case CHA_AlText:
		case CHA_InText:
		    lparse_unspec(rmptr + 3,session_ptr_p,id_ptr);
		break;

		case CHA_AlStat:
		    lparse_alstat(rmptr + 3,session_ptr_p);
		break;

		default:
		    rx_spi_unspec(rmptr + 3,session_ptr_p,id_ptr);
		break;
		}
	    break;

	default:
	    rx_spi_unspec(rmptr+3,session_ptr_p,id_ptr);
	break;
	}
    if (par_info_ptr->status == SINGLE_TUNED)
	par_info_ptr->status = IDLE;
    }
}

/***************************************************************************
*
* ROUTINE:mirror_rsp
*
* WRITTEN BY: Juergen Barthel                         ON:20.09.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine processes the MIRROR RESPONSE from CMS
*
****************************************************************************
* RETURNED VALUE: SUCCESS if the received MIRROR_RSP was a valid response to
*			  a regular MIRROR_REQ
*
*		  LIFETICK_REC if the received MIRROR_RSP was a valid
*			       response to a MIRROR_REQ command used for
*			       a lifetick
*
*		  FAIL if the received MIRROR_RSP is invalid
*
* EXTERNAL REFERENCES:
*
* Identifier       r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
mirror_rsp(MirrorRsp *rmptr,Session **session_ptr_p)
{
u_16    *info_ptr;
u_16	num_of_info;
u_16    i;
u_8	ret = TRUE;


/* check mirror info */
info_ptr    = &(rmptr->mirror_info[0]);
num_of_info = (u_16)(MIRROR_NUM +
	(((i_16 )(rmptr->trans_hd.trans_len) - (i_16)sizeof(MirrorRsp) )/2));
for (i = 0;i < num_of_info;i++,info_ptr++)
    {
    if ((*session_ptr_p)->mirror.info[i] != *info_ptr)
	{
	ret = FALSE;
	break;
	}
    }

/* check if received mirror is a regular- or a lifetick */
if (rmptr->trans_hd.src_id == DIR_SERVER)
    {
    if ((*session_ptr_p)->dest_id == DIR_SERVER)
	{
	free_session(session_ptr_p);
	}
    if (ret == TRUE)
	return(SUCCESS);
    else
	return(FAIL);
    }
else
    {
    if (ret == TRUE)
	{
	if ((*session_ptr_p)->mirror.life_status == ENABLE)
	    {
	    (*session_ptr_p)->mirror.status = LIFETICK_REC;
	    (*session_ptr_p)->mirror.time   = *bios_clocktick;
	    return(LIFETICK_REC);
	    }
	else
	    {
	    (*session_ptr_p)->mirror.status = DISABLE;
	    (*session_ptr_p)->mirror.time   = *bios_clocktick;
	    return(LIFETICK_REC);
	    }
	}
    else
	{
	/* if lifetick failed call rx_error, send DISCONNECT_REQ *
	 * and free the session tale */
	rx_error(LIFETICK_FAILED,(*session_ptr_p)->con_id);
	disconnect_req((*session_ptr_p)->con_id);
	free_session(session_ptr_p);
	return(FAIL);
	}
    }
}

/***************************************************************************
*
* ROUTINE:directory_rsp
*
* WRITTEN BY: Juergen Barthel                         ON:03.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine processes the DIRECTORY RESPONSE of CMS
*		DIRECTORY SERVER
*
****************************************************************************
* RETURNED VALUE: on success a pointer to a complete server table
*
*		  NULL if the servertable is incomplete or the command failed
*
* EXTERNAL REFERENCES:
*
* Identifier       r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
TblofServ
*directory_rsp(DirRsp *rmptr,Session **session_ptr_p)
{
PortTbl 	*port_ptr;
ServerTbl	*serv_tbl_ptr;
DirInfo		*dir_info_ptr;
u_16		i;
u_16		j;
u_16		serv_count;

/* search corresponding port table */
port_ptr = find_port_tbl((*session_ptr_p)->port_id);

/* check if a message was lost */
if ((rmptr->actual - port_ptr->tos.actual) != 1)
    {
    /* if the received message is the last DIRECTORY_RSP messsage, free *
     * the session */
    if (rmptr->actual == rmptr->total)
	free_session(session_ptr_p);
    return(NULL);
    }

/* init counting of the servers */
if (rmptr->actual == 1)
    port_ptr->tos.num_of_serv = 0;

serv_count = ((rmptr->trans_hd.trans_len-sizeof(TransHd)-4)/sizeof(DirInfo))
						+ port_ptr->tos.num_of_serv;
/* copy server info to server table */
serv_tbl_ptr = &(port_ptr->tos.server_tbl[0]);
dir_info_ptr = &(rmptr->dir_info[0]);

for (i = port_ptr->tos.num_of_serv;
     i < serv_count;i++,serv_tbl_ptr++,dir_info_ptr++)
    {
    serv_tbl_ptr->serv_id         = dir_info_ptr->serv_id;
    serv_tbl_ptr->num_connections = dir_info_ptr->num_connections;

    for (j = 0;j < DESCR_CHAR;j++)
	serv_tbl_ptr->serv_descr[j] = (c_8) dir_info_ptr->serv_descr[j];

    serv_tbl_ptr->compat_low  = dir_info_ptr->compat_low;
    serv_tbl_ptr->compat_high = dir_info_ptr->compat_high;
    }

port_ptr->tos.num_of_serv = serv_count;
port_ptr->tos.actual = rmptr->actual;

/* check if directory response is complete */
if (rmptr->actual == rmptr->total)
    {
    port_ptr->tos.total = rmptr->total;
    port_ptr->tos.language = rmptr->language;
    free_session(session_ptr_p);
    return(&(port_ptr->tos));
    }
return(NULL);
}

/***************************************************************************
*
* ROUTINE:raw_enable_rsp
*
* WRITTEN BY: Juergen Barthel                         ON:08.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine processes the RAW_ENABLE RESPONSE command
*
****************************************************************************
* RETURNED VALUE:on succees nonezero
*
* EXTERNAL REFERENCES:
*
* Identifier       r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
raw_enable_rsp(RawEnRsp *rmptr,Session **session_ptr_p)
{
free_session(session_ptr_p);
return(rmptr->rsp);
}

/***************************************************************************
*
* ROUTINE:connect_rsp
*
* WRITTEN BY: Juergen Barthel                         ON:08.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine processes the CONNECT RESPONSE command
*
****************************************************************************
* RETURNED VALUE:on succees it returns the window size of the new connection
*		 else
*		 FAIL if the connection was denied by the server
*
*		 INCOMPATIBLE_SERVER if the server is not compatible to the
*				     library
*
* EXTERNAL REFERENCES:
*
* Identifier       r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
connect_rsp(ConRsp *rmptr,Session **session_ptr_p)
{
u_8	com_low;
u_8	com_high;


if (rmptr->ret == 0)
    {
    free_session(session_ptr_p);
    return(FAIL);
    }
else
    {
    switch(rmptr->trans_hd.src_id)
	{
	case PAR_SERVER:
	default:
	    com_low  = PAR_COMPAT_LOW;
	    com_high = PAR_COMPAT_HIGH;
	break;

	}

    /* establish session, if
       CMS compat_high == LIB compat_high  AND
       CMS compat_low  <= LIB compat_low */
    if ((rmptr->compat_high != com_high) ||
	(rmptr->compat_low  >  com_low))
	{
	disconnect_req((*session_ptr_p)->con_id);
	free_session(session_ptr_p);
	return(INCOMPATIBLE_SERVER);
	}

    (*session_ptr_p)->window.size = rmptr->window;
    /* subtract length of CONNECT_REQ and MIRROR_REQ from window_size,
       because they are not checked by library
    */
    (*session_ptr_p)->window.count =
		(FRAMES_PER_S * (*session_ptr_p)->window.size) - 22;
    if ((*session_ptr_p)->mirror.life_status == ENABLE)
	{
	(*session_ptr_p)->mirror.status = LIFETICK_REC;
	(*session_ptr_p)->mirror.time 	= *bios_clocktick;
	}
    else
	{
	(*session_ptr_p)->mirror.status = DISABLE;
	(*session_ptr_p)->mirror.time 	= *bios_clocktick;
	}
    (*session_ptr_p)->window_time 	= *bios_clocktick;
    return(rmptr->window);
    }
}

/***************************************************************************
*
* ROUTINE:tick_rsp
*
* WRITTEN BY: Juergen Barthel                         ON:13.05.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine processes the TICK RESPONSE from CMS
*
****************************************************************************
* RETURNED VALUE: SUCCESS on succees, else
*
*		  FAIL
*
* EXTERNAL REFERENCES:
*
* Identifier       r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16
tick_rsp(TickRsp *rmptr,Session **session_ptr_p)
{
u_16    *tick_info_ptr;
u_16    num_of_info;
u_16	i;


tick_info_ptr = &(rmptr->tick_info[0]);
num_of_info = (u_16)(TICK_NUM +
	(((i_16 )(rmptr->trans_hd.trans_len) - (i_16)sizeof(TickRsp) )/2));

/* check tick info */
for (i = 0;i < num_of_info;i++,tick_info_ptr++)
    {
    if ((*session_ptr_p)->tick_info[i] != *tick_info_ptr)
	return(FAIL);
    }
return(SUCCESS);
}

/***************************************************************************
*
* ROUTINE:par_list_rsp
*
* WRITTEN BY: Juergen Barthel                         ON:08.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine processes the PAR_LIST RESPONSE command
*
****************************************************************************
* RETURNED VALUE:on succees it returns a pointer to a ParList else
*
*		 NULL
*
* EXTERNAL REFERENCES:
*
* Identifier       r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
ParList
*par_list_rsp(ListRsp *rmptr,Session **session_ptr_p)
{
IdentTyp	*ident_ptr;
ListInfo	*list_ptr;
u_16		par_count;
u_16		i;

/* check if a message was lost */
if ((rmptr->actual - (*session_ptr_p)->par.list.actual) != 1)
    return(NULL);

/* init counting of the msg IDs */
if (rmptr->actual == 1)
    (*session_ptr_p)->par.list.num_of_par = 0;

par_count = ((rmptr->trans_hd.trans_len - 10) / sizeof(ListInfo)) +
				     (*session_ptr_p)->par.list.num_of_par;

/* check if there are more elements in the response then can be stored */
if (par_count > MAX_PAR_IDS)
    {
    /* there are more elements in the response then can be stored *
     * therefore limit # of elements to the max. storeable number */
    par_count = MAX_PAR_IDS;
    }

list_ptr  = &(rmptr->list_info[0]);
ident_ptr = &((*session_ptr_p)->par.list.
	    ident[(*session_ptr_p)->par.list.num_of_par]);

for(i = (*session_ptr_p)->par.list.num_of_par;i < par_count;
	i++,ident_ptr++,list_ptr++)
   {
   ident_ptr->msg_id = list_ptr->msg_id_typ;
   c16_to_c8(list_ptr->ascii_id,ident_ptr->ascii_id,ASCII_ID_LENGTH);
   get_ascii_typ(list_ptr->msg_id_typ.MsgType,
		 &(ident_ptr->ascii_id[ASCII_ID_LENGTH]));
   }

(*session_ptr_p)->par.list.num_of_par = par_count;
(*session_ptr_p)->par.list.actual = rmptr->actual;

/* check if par_list response is complete */
if (rmptr->actual == rmptr->total)
    {
    (*session_ptr_p)->par.list.total = rmptr->total;
    return(&((*session_ptr_p)->par.list));
    }
return(NULL);
}

/***************************************************************************
*
* ROUTINE:misc_list_rsp
*
* WRITTEN BY: Juergen Barthel                         ON:08.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine processes the QUERY_LIST RESPONSE command
*
****************************************************************************
* RETURNED VALUE:on succees it returns a pointer to a MiscList else
*
*		 NULL
*
* EXTERNAL REFERENCES:
*
* Identifier       r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
MiscList
*misc_list_rsp(MListRsp *rmptr,Session **session_ptr_p)
{
IdentTyp	*ident_ptr;
ListInfo	*list_ptr;
u_16		misc_count;
u_16		i;

/* check if a message was lost */
if ((rmptr->actual - (*session_ptr_p)->misc.list.actual) != 1)
    return(NULL);

/* init counting of the msg IDs */
if (rmptr->actual == 1)
    (*session_ptr_p)->misc.list.num_of_par = 0;

misc_count = ((rmptr->trans_hd.trans_len - 10) / sizeof(ListInfo)) +
				     (*session_ptr_p)->misc.list.num_of_par;

/* check if there are more elements in the response then can be stored */
if (misc_count > MAX_MISCL_IDS)
    {
    /* there are more elements in the response then can be stored *
     * therefore limit # of elements to the max. storeable number */
    misc_count = MAX_PAR_IDS;
    }

list_ptr  = &(rmptr->list_info[0]);
ident_ptr = &((*session_ptr_p)->misc.list.
			ident[(*session_ptr_p)->misc.list.num_of_par]);

for (i=(*session_ptr_p)->misc.list.num_of_par;i < misc_count;
				i++,ident_ptr++,list_ptr++)
   {
   ident_ptr->msg_id = list_ptr->msg_id_typ;
   c16_to_c8(list_ptr->ascii_id,ident_ptr->ascii_id,ASCII_ID_LENGTH);
   get_ascii_typ(list_ptr->msg_id_typ.MsgType,
		 &(ident_ptr->ascii_id[ASCII_ID_LENGTH]));
   }

(*session_ptr_p)->misc.list.num_of_par = misc_count;
(*session_ptr_p)->misc.list.actual     = rmptr->actual;

/* check if misc_list response is complete */
if (rmptr->actual == rmptr->total)
    {
    (*session_ptr_p)->misc.list.total = rmptr->total;
    return(&((*session_ptr_p)->misc.list));
    }
return(NULL);
}

/***************************************************************************
*
* ROUTINE:query_list_rsp
*
* WRITTEN BY: Juergen Barthel                         ON:08.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine processes the QUERY_LIST RESPONSE command
*
****************************************************************************
* RETURNED VALUE:on succees it returns a pointer to a QueryList else
*
*		 NULL
*
* EXTERNAL REFERENCES:
*
* Identifier       r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
QueryList
*query_list_rsp(QListRsp *rmptr,Session **session_ptr_p)
{
u_16		*query_id_ptr;
u_16		*id_ptr;
u_16		query_count;
u_16		i;

/* check if a message was lost */
if ((rmptr->actual - (*session_ptr_p)->query.actual) != 1)
    return(NULL);

/* init counting of the msg IDs */
if (rmptr->actual == 1)
    (*session_ptr_p)->query.num_of_par = 0;

query_count = ((rmptr->trans_hd.trans_len - 10) / sizeof(u_16)) +
				     (*session_ptr_p)->query.num_of_par;

/* check if there are more elements in the response then can be stored */
if (query_count > MAX_QUERYL_IDS)
    {
    /* there are more elements in the response then can be stored *
     * therefore limit # of elements to the max. storeable number */
    query_count = MAX_PAR_IDS;
    }

/* copy the query ids into the session table element query.msg_id */
query_id_ptr = &(rmptr->query_id[0]);
id_ptr=&((*session_ptr_p)->query.query_id[(*session_ptr_p)->query.num_of_par]);

for (i=(*session_ptr_p)->query.num_of_par;i < query_count;i++)
   *id_ptr++ = *query_id_ptr++;

(*session_ptr_p)->query.num_of_par = query_count;
(*session_ptr_p)->query.actual = rmptr->actual;

/* check if query_list response is complete */
if (rmptr->actual == rmptr->total)
    {
    (*session_ptr_p)->query.total = rmptr->total;
    return(&((*session_ptr_p)->query));
    }
return(NULL);
}

/***************************************************************************
*
* ROUTINE:tune_rsp
*
* WRITTEN BY: Juergen Barthel                         ON:09.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine processes the TUNE RESPONSE command
*
****************************************************************************
* RETURNED VALUE: pointer to a RspList
*
* EXTERNAL REFERENCES:
*
* Identifier       r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
RspList *tune_rsp(TuneRsp *rmptr,Session **session_ptr_p)
{
ParInfo		*par_info_ptr;
RspInfo		*tune_rsp_info_ptr;
u_16		*tune_info_ptr;
u_16		rest_of_info;
i_16		ret;
u_8		i;

rsp_list.num_of_par = 0;

/* check number of message ids in the tune response message */
rest_of_info = (rmptr->trans_hd.trans_len - 10)/2;

tune_rsp_info_ptr = rsp_list.info_ptr = &rsp_info[0];
tune_info_ptr     = &(rmptr->tune_info[0]);
for (i = 0;rest_of_info >= 7;i++,tune_rsp_info_ptr++)
    {
    /* enter the message id into the response list */
    tune_rsp_info_ptr->msg_id = *((MsgIdTyp *)tune_info_ptr);

    /* search the corresponding ParInfo element */
    ret =
       search_info(session_ptr_p,&(tune_rsp_info_ptr->msg_id),&par_info_ptr);

    /* set the tune_info_ptr to the Mpb Hd */
    rest_of_info  -= (sizeof(MsgIdTyp)/2);
    tune_info_ptr += (sizeof(MsgIdTyp)/2);
    if ((*tune_info_ptr != 0) && (ret == 1))
	{
	/* tune suceeded, therefor rsp = 1 */
	tune_rsp_info_ptr->rsp = 1;
	/* enter the Mpb Hd in the corsponding ParInfo element */
	par_info_ptr->mpb_hd = *tune_info_ptr;
	if (rmptr->cmd == TUNE_RSP)
	    par_info_ptr->status = TUNED;
	else
	    par_info_ptr->status = SINGLE_TUNED;
	}
    else
	{
	if (ret == 1)
	    par_info_ptr->status = IDLE;

	/* tune failed, therefor rsp = 0 */
	tune_rsp_info_ptr->rsp = 0;
	}

    /* adjust tune_info_ptr by calculating the position of the next Msg Id */
    /* step over Mpb Hd */
    rest_of_info--;
    tune_info_ptr++;
    /* step over the app_rec_len/rec_id word and the appended words */
    rest_of_info -= ((HIGHBYTE(*tune_info_ptr)) + 1);
    tune_info_ptr += ((HIGHBYTE(*tune_info_ptr)) + 1);
    rsp_list.num_of_par++;
    }
return(&rsp_list);
}

/***************************************************************************
*
* ROUTINE:tune_type_rsp
*
* WRITTEN BY: Juergen Barthel                         ON:09.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine processes the TUNE RESPONSE command
*
****************************************************************************
* RETURNED VALUE: pointer to a RspList
*
* EXTERNAL REFERENCES:
*
* Identifier       r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
RspList *tune_type_rsp(TuneTypeRsp *rmptr,Session **session_ptr_p)
{
ParInfo		*par_info_ptr;
RspInfo		*tune_rsp_info_ptr;
u_16		*tune_info_ptr;
u_16		rsp_id_num;
u_16		j;


/* check if a message was lost */
if ((rmptr->actual - tune_type_actual) != 1)
    {
    tune_type_actual = 0;
    tune_type_busy_flag = FALSE;
    tune_type_flag = IDLE;
    tune_type_rsp_list.num_of_par = 0;
    return(NULL);
    }

if (rmptr->actual == 1)
    {
    tune_type_rsp_list.num_of_par = 0;
    }

tune_rsp_info_ptr = &tune_type_rsp_info[tune_type_rsp_list.num_of_par];
tune_type_rsp_list.info_ptr = &tune_type_rsp_info[0];

/* calculate number of msg ids in the message */
rsp_id_num = (rmptr->trans_hd.trans_len
	     - (sizeof(TuneTypeRsp) - ((MAX_TUNE_REQ-1) << 1))
	     - (rmptr->app_rec_len << 1) )
	     / (sizeof(MsgIdTyp) + 2);

tune_info_ptr = rmptr->tune_info + rmptr->app_rec_len;
for (j = 0;j < rsp_id_num;j++,tune_rsp_info_ptr++)
    {
    /* if there is space in one of the lists, add *
     * the message Id there */
    if(search_info(session_ptr_p,(MsgIdTyp *)tune_info_ptr,&par_info_ptr)==0)
	{
	if ((*session_ptr_p)->par.list.num_of_par < MAX_PAR_IDS)
	    {
	    par_info_ptr = &((*session_ptr_p)->par.
				info[(*session_ptr_p)->par.list.num_of_par]);
	    (*session_ptr_p)->par.list.ident[(*session_ptr_p)->par.list.
	    num_of_par].msg_id = *((MsgIdTyp *)tune_info_ptr);

	    ((*session_ptr_p)->par.list.num_of_par)++;
	    }
	else if ((*session_ptr_p)->misc.list.num_of_par < MAX_MISCL_IDS)
	    {
	    par_info_ptr = &((*session_ptr_p)->misc.info[
				    (*session_ptr_p)->misc.list.num_of_par]);
	    (*session_ptr_p)->misc.list.ident[(*session_ptr_p)->misc.list.
	    num_of_par].msg_id = *((MsgIdTyp *)tune_info_ptr);

	    ((*session_ptr_p)->misc.list.num_of_par)++;
	    }

	else
	    {
	    /* no more space available in the lists */
	    break;
	    }
	}

    /* enter the message id into the response list */
    tune_rsp_info_ptr->msg_id = *((MsgIdTyp *)tune_info_ptr);

    /* set the tune_info_ptr to the Mpb Hd */
    tune_info_ptr += (sizeof(MsgIdTyp)/2);
    if (*tune_info_ptr != 0)
	{
	/* tune suceeded, therefor rsp = 1 */
	tune_rsp_info_ptr->rsp = 1;
	/* enter the Mpb Hd in the corsponding ParInfo element */
	par_info_ptr->mpb_hd = *tune_info_ptr;
	par_info_ptr->level  = tune_type_flag;

	if (rmptr->cmd == TUNE_TYPE_RSP)
	    par_info_ptr->status = TUNED;
	else
	    par_info_ptr->status = SINGLE_TUNED;
	}
    else
	{
	par_info_ptr->status = IDLE;

	/* tune failed, therefor rsp = 0 */
	tune_rsp_info_ptr->rsp = 0;
	}
    /* set the tune info pointer to the next msg_id */
    tune_info_ptr++;
    tune_type_rsp_list.num_of_par++;
    }

/* check if TUNE_TYPE response is complete */
if (rmptr->actual == rmptr->total)
    {
    tune_type_actual = 0;
    tune_type_busy_flag = FALSE;
    tune_type_flag = IDLE;
    return(&tune_type_rsp_list);
    }
tune_type_actual++;
return(NULL);
}

/***************************************************************************
*
* ROUTINE:detune_rsp
*
* WRITTEN BY: Juergen Barthel                         ON:09.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine processes the DETUNE RESPONSE command
*
****************************************************************************
* RETURNED VALUE: pointer to a RspList
*
* EXTERNAL REFERENCES:
*
* Identifier       r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
RspList *detune_rsp(DetuneRsp *rmptr,Session **session_ptr_p)
{
ParInfo		*par_info_ptr;
MsgIdTyp	*id_ptr;
RspInfo		*detune_rsp_info_ptr;
DetuneInfo	*detune_info_ptr;
u_8		found;
u_16		i;
u_16		j;

/* check number of message ids in the detune response message */
rsp_list.num_of_par = (rmptr->trans_hd.trans_len-10)/sizeof(DetuneInfo);

/* check the detune response */
detune_rsp_info_ptr = rsp_list.info_ptr = &rsp_info[0];
detune_info_ptr     = &(rmptr->detune_info[0]);
for (j=0;j < rsp_list.num_of_par;j++,detune_rsp_info_ptr++,detune_info_ptr++)
    {
    found = FALSE;

    /* search the corresponding Msg Id */
    id_ptr       = &((*session_ptr_p)->par.list.ident[0].msg_id);
    par_info_ptr = &((*session_ptr_p)->par.info[0]);
    for (i=0;i < (*session_ptr_p)->par.list.num_of_par;
						i++,id_ptr++,par_info_ptr++)
	{
	if (par_info_ptr->status == TUNED)
	    {
	    /* find the corresponding parameter in the message info table */
	    if (par_info_ptr->mpb_hd == detune_info_ptr->mpb_hd)
		{
		found = TRUE;
		break;
		}
	    }
	}
    if (found == FALSE)
	{
	id_ptr       = &((*session_ptr_p)->misc.list.ident[0].msg_id);
	par_info_ptr = &((*session_ptr_p)->misc.info[0]);
	for (i=0;i < (*session_ptr_p)->misc.list.num_of_par;
						i++,id_ptr++,par_info_ptr++)
	    {
	    if (par_info_ptr->status == TUNED)
		{
	     /* find the corresponding parameter in the message info table */
		if (par_info_ptr->mpb_hd == detune_info_ptr->mpb_hd)
		    {
		    break;
		    }
		}
	     }
	}
    /* enter the message id into the response list */
    detune_rsp_info_ptr->msg_id = *id_ptr;

    /* enter the response into the response list */
    if (detune_info_ptr->rsp != 0)
	{
	/* detune suceeded, therefor rsp = 1 */
	detune_rsp_info_ptr->rsp = 1;
	par_info_ptr->mpb_hd     = 0;
	par_info_ptr->status     = IDLE;
	}
    else
	{
	/* detune failed, therefor rsp = 0 */
	detune_rsp_info_ptr->rsp = 0;
	}
    }
return(&rsp_list);
}

/***************************************************************************
*
* ROUTINE:detune_all_rsp
*
* WRITTEN BY: Juergen Barthel                         ON:09.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine processes the DETUNE ALL RESPONSE command
*
****************************************************************************
* RETURNED VALUE: SUCCESS on succees else
*
*		  FAIL
*
* EXTERNAL REFERENCES:
*
* Identifier       r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16 detune_all_rsp(DetAllRsp *rmptr,Session **session_ptr_p)
{
ParInfo		*par_info_ptr;
IdentTyp	*ident_ptr;
u_16		i;

/* check if detune all was successfull */
if (rmptr->rsp != 1)
    return(FAIL);

/* clear all matching tune entries */
ident_ptr = &((*session_ptr_p)->par.list.ident[0]);
par_info_ptr = &((*session_ptr_p)->par.info[0]);
for (i = 0; i < (*session_ptr_p)->par.list.num_of_par;i++,par_info_ptr++,
							  ident_ptr++)
    {
    if (par_info_ptr->status != IDLE)
	{
	if (rmptr->msg_type == 0)
	    {
	    par_info_ptr->mpb_hd = 0;
	    par_info_ptr->status = IDLE;
	    }
	else if (rmptr->msg_type == ident_ptr->msg_id.MsgType)
	    {
	    par_info_ptr->mpb_hd = 0;
	    par_info_ptr->status = IDLE;
	    }
	}
    }
ident_ptr = &((*session_ptr_p)->misc.list.ident[0]);
par_info_ptr = &((*session_ptr_p)->misc.info[0]);
for (i = 0;i < (*session_ptr_p)->misc.list.num_of_par;i++,par_info_ptr++,
							  ident_ptr++)
    {
    if (par_info_ptr->status != IDLE)
	{
	if (rmptr->msg_type == 0)
	    {
	    par_info_ptr->mpb_hd = 0;
	    par_info_ptr->status = IDLE;
	    }
	else if (rmptr->msg_type == ident_ptr->msg_id.MsgType)
	    {
	    par_info_ptr->mpb_hd = 0;
	    par_info_ptr->status = IDLE;
	    }
	}
    }
return(SUCCESS);
}

/***************************************************************************
*
* ROUTINE:disconnect_rsp
*
* WRITTEN BY: Juergen Barthel                         ON:09.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine processes the DISCONNECT RESPONSE command
*
****************************************************************************
* RETURNED VALUE: SUCCESS on suceess else
*
*		  FAIL
*
* EXTERNAL REFERENCES:
*
* Identifier       r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16 disconnect_rsp(DisconRsp *rmptr,Session **session_ptr_p)
{
/* check if disconnect request was successfull */
if (rmptr->rsp == 1)
    {
    free_session(session_ptr_p);
    return(SUCCESS);
    }
else
    return(FAIL);
}

/***************************************************************************
*
* ROUTINE:cms_tick_rsp
*
* WRITTEN BY: Juergen Barthel                         ON:12.06.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine processes a incomming TICK REQUEST command and
*		sends the TICK RESPONSE to the CMS
*
****************************************************************************
*
* RETURNED VALUE: OUT_OF_WINDOW_SIZE if window size is suspended
*
*                 else the return value of "write_msg()" whitch is:
*
*		  SUCCESS if command will be transmitted,
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it,
*
*		  HW_HS_MISSING if the hardware handshake is missing,
*
*				NOTE: this indicates that the outputbuffer
*				was flashed and the specified message will
*				not be transmitted
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16 cms_tick_rsp(TickReq *rmptr,Session **session_ptr_p)
{
u_16	i;

/* set the TICK RESPONSE command */
cmstick_rsp.trans_hd.trans_len = rmptr->trans_hd.trans_len;
cmstick_rsp.trans_hd.dest_id   = rmptr->trans_hd.src_id;
cmstick_rsp.trans_hd.src_id    = rmptr->trans_hd.dest_id;
cmstick_rsp.cmd 	       = TICK_RSP;

/* check window size */
if ((*session_ptr_p)->window.check == ENABLE)
    {
    if ((*session_ptr_p)->window.count < cmstick_rsp.trans_hd.trans_len)
	return(OUT_OF_WINDOW_SIZE);

    (*session_ptr_p)->window.count -= cmstick_rsp.trans_hd.trans_len;
    }

/* copy tick information from received message */
for (i = 0;(i_16)(2 * i) < ((i_16)rmptr->trans_hd.trans_len - 8);i++)
    cmstick_rsp.tick_info[i] = rmptr->tick_info[i];

return(write_msg((*session_ptr_p)->port_id,(u_16 *)&cmstick_rsp));
}

/***************************************************************************
*
* ROUTINE:cms_mirror_rsp
*
* WRITTEN BY: Juergen Barthel                         ON:12.06.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine processes a incomming MIRROR REQUEST command and
*		sends the MIRROR RESPONSE to the CMS
*
****************************************************************************
*
* RETURNED VALUE: the return value of "write_msg()" whitch is:
*
*		  SUCCESS if command will be transmitted,
*
*		  FAIL if the specified message has an incorrect length,
*
*                 OUT_OF_TXBUFFER if outputbuffer is full and therefor the
*		                  specified message was not written to it,
*
*		  HW_HS_MISSING if the hardware handshake is missing,
*
*				NOTE: this indicates that the outputbuffer
*				was flashed and the specified message will
*				not be transmitted
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
i_16 cms_mirror_rsp(MirrorReq *rmptr,Session **session_ptr_p)
{
u_16	*mir_req_wrd_ptr;
u_16	*mir_rsp_wrd_ptr;
u_16	num_of_wrd;
u_16	i;

/* set the MIRROR RESPONSE command */
mir_rsp.cmd = MIRROR_RSP;
/* copy length, source and destination from request to response */
mir_rsp.trans_hd.trans_len = rmptr->trans_hd.trans_len;
mir_rsp.trans_hd.dest_id   = rmptr->trans_hd.src_id;
mir_rsp.trans_hd.src_id    = rmptr->trans_hd.dest_id;

/* copy mirror info from request to response */
if ((num_of_wrd = (rmptr->trans_hd.trans_len-sizeof(TransHd)-2) / 2) != 0)
    {
    mir_req_wrd_ptr = &(rmptr->mirror_info[0]);
    mir_rsp_wrd_ptr = &(mir_rsp.mirror_info[0]);
    for (i = 0;i < num_of_wrd;i++)
	*mir_rsp_wrd_ptr++ = *mir_req_wrd_ptr++;
    }

/* send MIRROR_RSP to CMS */
return(write_msg((*session_ptr_p)->port_id,(u_16 *)&mir_rsp));
}

/***************************************************************************
*
* ROUTINE:get_ascii_typ
*
* WRITTEN BY: Juergen Barthel                         ON:09.04.91
* CHANGED BY:                                         ON:
*
****************************************************************************
*
* DESCRIPTION:this routine writes an ASCII identifier to a pointed location
*
****************************************************************************
* RETURNED VALUE:none
*
* EXTERNAL REFERENCES:
*
* Identifier      r/w  Comment
* --------------------------------------------------------------------------
*
****************************************************************************/
static void get_ascii_typ(u_16 msg_type,c_8 *ptr)
{
switch (msg_type)
    {
    case TYP_SpiCW:
	strcpy(ptr,"-CW");
    break;

    case TYP_SpiWS:
	strcpy(ptr,"-WS");
    break;

    case TYP_SpiNU:
	strcpy(ptr,"-NU");
    break;

    case TYP_SpiMX:
	strcpy(ptr,"-MX");
    break;

    default:
	strcpy(ptr,"   ");
    break;
    }
}
