/********************************************************HP*03-5310*********
*
* $Author: prakt_fw $
* $Date: 92/12/03 10:37:25 $
* $Revision: 1.10 $
*
****************************************************************************
*
* CONTENTS :    demo.c - MECIF PC demo program for TURBO C
*	     	         ---------------------------------
*
* Shows access to the following data of the Component Monitoring System
* via RS232:
* 		  ECG wave and wave support,
*		  HR and PRESSURE numerics,
*		  and ALARM status
*
* Data is shown on the PC display.
*
***************************************************************************


		(C) Copyright Hewlett-Packard GmbH 1992

			  All rights reserved


     Hewlett-Packard makes no warranty of any kind with regard to this 
     software, including, but not limited to, the implied warranties of 
     merchantability and fitness for a particular purpose.
     Hewlett-Packard shall not be liable for errors contained herein or
     direct, indirect, special, incidental or consequential damages in 
     connection with the furnishing, performance or use of this material.

*/



/*-SYSTEM INCLUDES--------------------------------------------------------*/

#include <stdio.h>
#include <conio.h>
#include <graphics.h>
#include <stdlib.h>
#include <time.h>
#include <mem.h>
#include <string.h>


/*-MODULE INCLUDES--------------------------------------------------------*/

#include <meciflib.h>
#include <database.h>

/*-EXTERNAL FUNCTIONS-----------------------------------------------------*/


/*-EXTERNAL VARIABLES-----------------------------------------------------*/


/*-DEFINES----------------------------------------------------------------*/

#define TRUE    	1
#define FALSE   	0
#define TIMEOUT 	6
#define SYSTOLIC 	0X01
#define DIASTOLIC 	0X02
#define MEAN		0X03


/*-TYPEDEFS---------------------------------------------------------------*/


/*-GLOBAL FUNCTIONS-------------------------------------------------------*/


/*-GLOBAL VARIABLES-------------------------------------------------------*/


/*-STATIC FUNCTIONS-------------------------------------------------------*/

static void     init_tune_list(void);
static void     init_user_graphics(void);
static void     close_graphics(void);


/*-STATIC VARIABLES-------------------------------------------------------*/

static i_16 		graphdriver;
static i_16 		graphmode;
static i_16		max_x;
static i_16		max_y;
static i_16 		yspan;
static i_16		*sample_start_ptr;
static i_16		*sample_ptr;
static LibTuneList	tune_list;	/* list of MsgIds to be tuned         */
static MsgIdTyp		msg_id_list[5];
static u_16		port_id;
static boolean		startup;
static boolean		restarting;
static boolean		shutdown;


/***************************************************************************
*
* ROUTINE:     main
*
****************************************************************************
*
* DESCRIPTION: Main routine.
*	       -------------
*
* 	- initializes the MECIF library
*	- configures COM1 for MECIF
*	- checks the connection by sending a MIRROR request
*	- connects to the Parameter Server
*	- requests the list of available parameters
*	- tunes on ECG wave and wave support, heart rate and invasive
*	   pressure numerics and the alarm status
*	- the received data is displayed
*	- when a key is pressed on the keyboard, all parameters are
*	   detuned, the connection to the Parameter Server is terminated
*	   and the COM1 is closed
*
* To check the proper connection to CMS, a timeout of 6 seconds is set
* for every command response.
*
***************************************************************************/
main()
{
ParList 	*list_ptr;
LibRet		*status;
time_t          time1;
time_t		time2;
u_16		con_id;
i_16 		i;
i_16 		port;
boolean         timeout_enable;
boolean		go_on;


/* initialize all user specific variables                                 */
init_tune_list();
port = 0;		/* select RS232 COM1 for this application         */
go_on = TRUE;           /* set start value for reading loop               */
startup = TRUE;         /* set startup sequence TRUE                      */
restarting = FALSE;     /* set restarting sequence FALSE                  */
shutdown = FALSE;       /* set shutdown sequence FALSE	                  */

/* initialize COM1 to run MECIF on it and set baudrate to 19200 baud      */
port_id = init_rs232port(port, 19200);
if (port_id == BAUDRATE_WRONG)
    {
    printf("ERROR equipment malfunction (wrong baudrate)!");
    exit(0);
    }
else if (port_id == PORT_WRONG)
    {
    printf("ERROR equipment malfunction (wrong port)!");
    exit(0);
    }

/* initialize graphic modes of the PC                                     */
init_user_graphics();

/* check the line with a mirror request 				  */
mirror_req(0,port_id,0,0);


/* read all the messages coming in from the CMS as long as no PC key is
   pressed.
   This is the begin of the reading loop.				  */
while (go_on == TRUE)
    {
    /* read next incoming message from CMS                                */
    status = rx_mecif();

    /* identify the kind of message that has been read.
       As soon as an expected response is given by the CMS, the next
       command is triggered 				                  */

    switch(status->cmd)
	{
	case MIRROR_RSP:
	    if (startup == TRUE)
		{
		/* connect to the CMS Parameter Server via COM1 using a
		   5 seconds lifetick period for the library and the CMS,
		   enable the window size control mechanism of the library
		   and store the returned connection number      	  */
		con_id = connect_req(port_id,SRC_ParServer,5,5,
				     ENABLE);

		startup = FALSE;            /* startup sequence finished  */
		restarting = FALSE;         /* restarting finished        */
		}
	    break;

	case CONNECT_RSP:
	    /* get a list of all MESSAGE IDs that are provided by the
	       Parameter Server within connection 'con_id'               */
	    par_list_req(con_id);
	    time(&time1);	           /* set time for timeout check */
	    timeout_enable = TRUE;         /* enable timeout checking    */
	    break;

	case PAR_LIST_RSP:
	    list_ptr = status->ret.pret;
	    if (list_ptr)       /* if parameter list is valid go on       */
		{
		/* search for the ALARM STATUS message ID in the list
		   of parameter IDs provided by the Parameter Server
		   we are connected to (the define statements used for
		   the IDs here can be found in the header file
		   "database.h") 					  */

		for (i = 0;i < list_ptr->num_of_par;i++)
		    {
		    if ((list_ptr->ident[i].msg_id.SourceId == SRC_AlMgr) &&
			(list_ptr->ident[i].msg_id.ChannelId == CHA_AlStat))
			{
			/* add the found ID to the tune_list              */
			msg_id_list[4] = list_ptr->ident[i].msg_id;
			tune_list.num_tune += 1;
			break;
			}
		    }

		/* Transmit a CONTINUOUS TUNE REQUEST for the HEART RATE
		   numeric message, the ECG WAVE message, the
		   ECG WAVE SUPPORT message, the PRESSURE NUMERIC
		   message with the IDs prepared in 'init_tune_list' and
		   the ALARM STATUS message that has just been added to
		   the tune_list                                          */
		tune_lib_req(con_id, &tune_list, CONTINUOUS);

		timeout_enable = FALSE;    	/* disable timeouts       */
		}
	    break;

	case DETUNE_ALL_RSP:
	    disconnect_req(con_id);
	    time(&time1);          	    /* set time for timeout check */
	    break;

	case DISCONNECT_RSP:
	    close_graphics();
	    close_port(port_id);
	    time(&time1);
	    go_on = FALSE;         	   /* set go_on to FALSE to leave */
	    break;		   	   /*  the reading loop 	  */
	}

    /*---- timeout check -------------------------------------------------*/
    if (timeout_enable == TRUE)
	{
	/* if time has elapsed then end the program and print an error
	    message on the screen     					  */

	/* Note: responses for MIRROR_REQ and CONNECT_REQ are already
		 checked by the MECIF library and in case of errors or
		 timeoutes rx_error() is called                           */
	if ((time(&time2) - time1) > TIMEOUT)
	    {
	    close_graphics();
	    close_port(port_id);
	    printf("ERROR equipment malfunction !");
	    break;
	    }
	}


    /*----- keyboard check to end reading loop ---------------------------*/
    if (kbhit() != 0)
	{
	if (shutdown == FALSE)
	    {
	    getch();

	    /* before ending the program, detune all previously tuned
	       messages 				                  */
	    detune_all_req(con_id,0);
	    timeout_enable = TRUE;     	   /* enable timeout check for    */
	    shutdown = TRUE;               /*  shutdown sequence          */
	    time(&time1);
	    }
	}

    /* ------- this is the end of the reading loop -----------------------*/
    }

exit(0);
return;
}


/***************************************************************************
*
* ROUTINE:     init_tune_list
*
****************************************************************************
*
* DESCRIPTION: Set up message IDs which are to be tuned.
*
* For the definition of the message ID fields see the header files
* DATABASE.H and PC_SPI.H.
*
***************************************************************************/
static void
init_tune_list(void)
{
/* set up the MESSAGE ID of a HEART RATE numeric                          */
msg_id_list[0].SourceId  = SRC_HeartModule;
msg_id_list[0].ChannelId = CHA_HeartRate;
msg_id_list[0].MsgType   = TYP_SpiNU;
msg_id_list[0].ChannelNo = 1;
msg_id_list[0].SourceNo  = 1;
msg_id_list[0].Layer     = 1;

/* set up the MESSAGE ID of a ECG WAVE          			  */
msg_id_list[1].SourceId  = SRC_ThreeChanEcg;
msg_id_list[1].ChannelId = CHA_Ecg;
msg_id_list[1].MsgType   = TYP_SpiCW;
msg_id_list[1].ChannelNo = 1;
msg_id_list[1].SourceNo  = 1;
msg_id_list[1].Layer     = 1;

/* set up the MESSAGE ID of an ECG WAVE SUPPORT message                   */
msg_id_list[2].SourceId  = SRC_ThreeChanEcg;
msg_id_list[2].ChannelId = CHA_Ecg;
msg_id_list[2].MsgType   = TYP_SpiWS;
msg_id_list[2].ChannelNo = 1;
msg_id_list[2].SourceNo  = 1;
msg_id_list[2].Layer     = 1;

/* set up the MESSAGE ID of a PRESSURE numeric                            */
msg_id_list[3].SourceId  = SRC_InvPress;
msg_id_list[3].ChannelId = CHA_PressNum;
msg_id_list[3].MsgType   = TYP_SpiNU;
msg_id_list[3].ChannelNo = 1;
msg_id_list[3].SourceNo  = 1;
msg_id_list[3].Layer     = 1;

tune_list.num_tune = 4;
tune_list.msg_id_ptr = &msg_id_list[0];
}

/***************************************************************************
*
* MECIF Library Interface
* -----------------------
*
* The functions 'rx_lib_...()' and 'rx_spi_...()' have to be part of the
* user application routines
*
* The following functions 'rx_lib_...()' are called by the library in case
* an incomming message is tuned with 'tune_lib_req()'.
*
***************************************************************************/

/***************************************************************************
*
* ROUTINE:     rx_lib_nu
*
****************************************************************************
*
* DESCRIPTION: Processes NUMERIC messages.
*
* Displays the heart rate and the invasive pressure numerics on the screen.
*
***************************************************************************/
void
rx_lib_nu(Numeric *nu_ptr)
{
i_16	i;
i_16	s;
i_16	d;
i_16	m;
u_8 	value[13];

switch(nu_ptr->msg_id.SourceId)
    {
    case SRC_InvPress:
	setviewport(max_x/2,textheight("t")*5,max_x,textheight("t")*6,1);
	clearviewport();
	setcolor(LIGHTRED);

	/* check if values are valid                                      */
	if ((nu_ptr->status & 0x80) == 0x80)
	    {
	    outtext(nu_ptr->label);
	    outtext("-?-");
	    }
	else
	    {
	    /* get systolic, diastolic and mean				  */
	    for (i = 0;i < nu_ptr->num_of_values;i++)
		{
		switch(nu_ptr->numeric[i].label_code & 0x03)
		    {
		    case SYSTOLIC:
			s = nu_ptr->numeric[i].value;
			break;

		    case DIASTOLIC:
			d = nu_ptr->numeric[i].value;
			break;

		    case MEAN:
			m = nu_ptr->numeric[i].value;
			break;
		    }
		}

	    /* convert the value(s) to a string and display this string
                on the screen                                             */
	    if ((nu_ptr->num_of_values) == 1)
		{
		/* mean only                                              */
		sprintf(value,"(%d)", m);
		}
	    else if ((nu_ptr->status & 0x40) == 0x40)
		{
		/* data is questionable					  */
		sprintf(value,"?%d/%d(%d)", s, d, m);
		}
	    else
		{
		sprintf(value,"%d/%d(%d)", s, d, m);
		}
	    outtext(nu_ptr->label);
	    outtext(value);
	    }
	break;

    case SRC_HeartModule:
	setviewport(0,textheight("t")*5,(max_x/2)-1,textheight("t")*6,1);
	clearviewport();
	setcolor(YELLOW);

	/* check if value is valid                                        */
	if ((nu_ptr->status & 0x80) == 0x80)
	    {
	    outtext(nu_ptr->label);
	    outtext("-?-");
	    }
	else
	    {
	    /* convert the value(s) to a string and display this string
	       on the screen                                              */
	    outtext(&(nu_ptr->label[0]));
	    sprintf(value,"%i",(i_16)nu_ptr->numeric[0].value);
	    outtextxy(72,0,value);
	    }
	break;
    }
}

/***************************************************************************
*
* ROUTINE:     rx_lib_ws
*
****************************************************************************
*
* DESCRIPTION: Processes WAVE SUPPORT messages.
*
* Shows the ECG lead label and the status (unplugged, parameter off).
*
***************************************************************************/
void
rx_lib_ws(WaveSupp *ws_ptr)
{
/* check if parameter is unplugged                                        */
switch ((ws_ptr->status) & 0xA000)
    {
    case 0X8000:   /* parameter is on, but module is unplugged            */
	/*  clear ECG wave viewport                                       */
	setviewport(0,textheight("t")*6,max_x,max_y-24,1);
	clearviewport();

	/* display the label and ECG status message on the screen         */
	setviewport(0,textheight("t")*9,textwidth("t")*4,
						textheight("t")*10,1);
	clearviewport();
	setcolor(YELLOW);
	outtextxy(0,0,ws_ptr->label);
	setviewport(0,max_y-textheight("t")-8,textwidth("t")*22,max_y-9,1);
	clearviewport();
	outtext("ECG module unplugged !");
	break;

    case 0XA000: /* parameter is on and module is plugged                 */
	/* clear message viewport                                         */
	setviewport(0,max_y-textheight("t")-8,max_x,max_y-9,1);
	clearviewport();

	/* display the label on the screen                                */
	setviewport(0,textheight("t")*9,textwidth("t")*4,
						textheight("t")*10,1);
	clearviewport();
	setcolor(YELLOW);
	outtextxy(0,0,ws_ptr->label);
	break;

    default: /* parameter is off                                          */
	/* clear ECG wave viewport                                        */
	setviewport(0,textheight("t")*6,max_x,max_y-24,1);
	clearviewport();

	/* display the ECG status message on the screen                   */
	setviewport(0,max_y-textheight("t")-8,textwidth("t")*22,max_y-9,1);
	clearviewport();
	setcolor(YELLOW);
        outtext("ECG Parameter off !");
	break;
    }
}

/***************************************************************************
*
* ROUTINE:     rx_lib_alstat
*
****************************************************************************
*
* DESCRIPTION: Processes WAVE SUPPORT messages.
*
* Prints the Alarm Status on the screen.
*
***************************************************************************/
void
rx_lib_alstat(AlStat *alstat_ptr)
{
setviewport(max_x/2,textheight("t")*3,max_x,textheight("t")*4,1);
setfillstyle(SOLID_FILL,WHITE);
clearviewport();
bar(0,0,max_x/2,textheight("t"));
setcolor(BLACK);
if (alstat_ptr->al_suspended == 0)
    outtextxy(1,1,"ALARMS ON");
else
    outtextxy(1,1,"ALARMS SUSPENDED");
}

/***************************************************************************
*
* ROUTINE:     rx_lib_cw
*
****************************************************************************
*
* DESCRIPTION: Processes COOKED WAVE messages.
*
* Displays the ECG wave on the screen.
*
***************************************************************************/
void
rx_lib_cw(CookedWave *cw_ptr)
{
static i_16	x_pos;
i_16		offset;
u_8  		i;

setviewport(0,textheight("t")*6,max_x,max_y-24,1);
offset = max_x - 20;

/* read the samples and display each third sample on the screen, else
   the wave will be to fast(NOTE: this is not a carefully designed
   speed reduction algorithm but a readable example !!)		          */
for (i = 0;i < cw_ptr->num_of_samples;i+=3)
    {
    /* erase old line                                                     */
    if (x_pos >= offset)
	{
	setcolor(BLACK);
	line(x_pos-offset,*(sample_start_ptr+(x_pos-offset)),
	     x_pos +1-offset,*(sample_start_ptr+x_pos+1-offset));
	}
    else
	{
	setcolor(BLACK);
	line(x_pos+20,*(sample_ptr+21),x_pos+21,*(sample_ptr+22));
	}

    if (x_pos == 0)
	sample_ptr = sample_start_ptr;
    else
	sample_ptr++;

    /* draw new line                                                      */
    *sample_ptr = yspan - ((cw_ptr->samples[i]) / (4095 / yspan));

    setcolor(YELLOW);
    if (x_pos == 0)
	{
	putpixel(0,*sample_ptr,YELLOW);
	x_pos++;
	}
    else
	{
	line(x_pos-1,*(sample_ptr-1),x_pos,*sample_ptr);
	if (x_pos < max_x)
	    x_pos++;
	else
	    x_pos = 0;
	}
    }
}

/***************************************************************************
*
* ROUTINE:     rx_error
*
****************************************************************************
*
* DESCRIPTION: Error Handling routine
*
* The following routine is called by the library in case of an error.
*
***************************************************************************/
void
rx_error(u_16 error_code,i_16 con_id)
{
u_8			error[60];
i_16	ret;

setcolor(WHITE);

settextstyle(DEFAULT_FONT,HORIZ_DIR,1);

setviewport(max_x-440,max_y-textheight("t")-8,max_x,max_y-9,1);
clearviewport();

switch(error_code)
    {
    case WINDOW_SIZE_OVERFLOW:
	sprintf(error,"window size overflow (RX RS232->TX MPB) "
	"in con_id %i.",con_id);
	break;

    case TX_BUFFER_OVERFLOW:
	sprintf(error,"TX buffer overflow (RX MPB->TX RS232) in con_id %i.",
	con_id);
	break;

    case HW_HANDSHAKE_MALFUNCTION:
	sprintf(error,"RS232 HW-handshake malfunction in con_id %i.",
	con_id);
	break;

    case MPB_FIFO_OVERFLOW:
	sprintf(error,"MPB RX FIFO overflow in con_id %i.",con_id);
	break;

    case PARITY_ERROR:
	sprintf(error,"RS232 RX parity error in con_id %i.",con_id);
	break;

    case NOTIFIC_ABORTED:
	sprintf(error,"Notification abortet in con_id %i.",con_id);

	/* restarting sequence */
	if (shutdown == FALSE)
	    {
	    mirror_req(0, port_id, 0, 0);
	    restarting = TRUE;
	    startup = TRUE;
	    }
	break;

    case RS232_RESTART_OV:
	sprintf(error,"RS232 restart because of overload in con_id %i.",con_id);
	break;

    case RS232_RESTART_OTHER:
	sprintf(error,"RS232 restart in con_id %i.",con_id);
	break;

    case LIFETICK_TIMEOUT:
	sprintf(error,"Lifetick timeout in con_id %i.",con_id);

	/* restarting sequence */
	if (shutdown == FALSE)
	    {
	    mirror_req(0, port_id, 0, 0);
	    restarting = TRUE;
	    startup = TRUE;
	    }
	break;

    case LIFETICK_FAILED:
	sprintf(error,"Lifetick failed in con_id %i.",con_id);

	/* restarting sequence */
	if (shutdown == FALSE)
	    {
	    mirror_req(0, port_id, 0, 0);
	    restarting = TRUE;
	    startup = TRUE;
	    }
	break;

    case NO_RSP_MIR_REQ:
	sprintf(error,"No response for MIRROR_REQ in con_id %i.",con_id);

	/* restarting sequence */
	if (shutdown == FALSE)
	    {
	    mirror_req(0, port_id, 0, 0);
	    restarting = TRUE;
	    startup = TRUE;
	    }
	break;

    case NO_RSP_CON_REQ:
	sprintf(error,"No response for CONNECT_REQ in con_id %i.",con_id);

	/* restarting sequence */
	if (shutdown == FALSE)
	    {
	    mirror_req(0, port_id, 0, 0);
	    restarting = TRUE;
	    startup = TRUE;
	    }
	break;

    case NO_RSP_DISCON_REQ:
	sprintf(error,"No response for DISCONNECT_REQ in con_id %i.",con_id);
	break;

    case MESSAGE_LOST:
	sprintf(error,"Mecif message lost in con_id %i.",con_id);
	break;

    case INVALID_COMMAND:
	sprintf(error,"Invalid Mecif command in con_id %i.",con_id);
	break;

    default :
	sprintf(error,"undefined error in con_id %i",con_id);
	break;

    }
outtext(error);
settextstyle(DEFAULT_FONT,HORIZ_DIR,2);
return;
}

/***************************************************************************
*
* Graphic Routines
*
***************************************************************************/

/***************************************************************************
*
* ROUTINE:     init_user_graphics
*
****************************************************************************
*
* DESCRIPTION: Initializes the graphic screen.
*
***************************************************************************/
static void
init_user_graphics(void)
{
detectgraph(&graphdriver,&graphmode);
registerbgidriver(EGAVGA_driver);
registerbgidriver(CGA_driver);
registerbgidriver(ATT_driver);
registerbgidriver(PC3270_driver);
registerbgidriver(Herc_driver);
registerbgidriver(IBM8514_driver);
initgraph(&graphdriver,&graphmode,"");

setlinestyle(SOLID_LINE,0,1);
setfillstyle(0,0);

max_x = getmaxx();
max_y = getmaxy();

sample_start_ptr = (i_16 *)malloc(sizeof(i_16)*max_x);
sample_ptr = sample_start_ptr;


memset(sample_start_ptr,0,2*max_x);

settextstyle(DEFAULT_FONT,HORIZ_DIR,2);
yspan = max_y - textheight("t")*6 - 24;
setcolor(CYAN);

setviewport(0,0,max_x,textheight("t"),1);
outtext("  HP Computer Interface Demonstration");

settextstyle(DEFAULT_FONT,HORIZ_DIR,1);
setviewport(0,max_y-textheight("t"),max_x,max_y,1);
outtext("For further information, please refer to HP programming guide");

settextstyle(DEFAULT_FONT,HORIZ_DIR,2);
}

/***************************************************************************
*
* ROUTINE:     close_graphics
*
****************************************************************************
*
* DESCRIPTION: End the graphic mode and restore the old CRT mode.
*
***************************************************************************/
static void
close_graphics(void)
{
restorecrtmode();
closegraph();
free(sample_start_ptr);
}
